# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import csv

import datasets


_CITATION = """
@InProceedings{DARWISH18.562,  author = {Kareem Darwish ,Hamdy Mubarak ,Ahmed Abdelali ,Mohamed Eldesouki ,Younes Samih ,Randah Alharbi ,Mohammed Attia ,Walid Magdy and Laura Kallmeyer},
title = {Multi-Dialect Arabic POS Tagging: A CRF Approach},
booktitle = {Proceedings of the Eleventh International Conference on Language Resources and Evaluation (LREC 2018)},
year = {2018},
month = {may},
date = {7-12},
location = {Miyazaki, Japan},
editor = {Nicoletta Calzolari (Conference chair) and Khalid Choukri and Christopher Cieri and Thierry Declerck and Sara Goggi and Koiti Hasida and Hitoshi Isahara and Bente Maegaard and Joseph Mariani and Hélène Mazo and Asuncion Moreno and Jan Odijk and Stelios Piperidis and Takenobu Tokunaga},
publisher = {European Language Resources Association (ELRA)},
address = {Paris, France},
isbn = {979-10-95546-00-9},
language = {english}
}
"""

_DESCRIPTION = """\
The Dialectal Arabic Datasets contain four dialects of Arabic, Etyptian (EGY), Levantine (LEV), Gulf (GLF), and Maghrebi (MGR). Each dataset consists of a set of 350 manually segmented and POS tagged tweets.
"""

_URL = "https://github.com/qcri/dialectal_arabic_resources/raw/master/"
_DIALECTS = ["egy", "lev", "glf", "mgr"]


class ArabicPosDialectConfig(datasets.BuilderConfig):
    """BuilderConfig for ArabicPosDialect"""

    def __init__(self, dialect=None, **kwargs):
        """

        Args:
            dialect: the 3-letter code string of the dialect set that will be used.
            Code should be one of the following: egy, lev, glf, mgr.
            **kwargs: keyword arguments forwarded to super.
        """
        super(ArabicPosDialectConfig, self).__init__(**kwargs)
        assert dialect in _DIALECTS, ("Invalid dialect code: %s", dialect)
        self.dialect = dialect


class ArabicPosDialect(datasets.GeneratorBasedBuilder):
    """POS-tagged Arabic tweets in four major dialects."""

    VERSION = datasets.Version("1.1.0")
    BUILDER_CONFIG_CLASS = ArabicPosDialectConfig
    BUILDER_CONFIGS = [
        ArabicPosDialectConfig(
            name=dialect,
            dialect=dialect,
            description=f"A set of 350 tweets in the {dialect} dialect of Arabic that have been manually segmented and POS tagged.",
        )
        for dialect in _DIALECTS
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "fold": datasets.Value("int32"),
                    "subfold": datasets.Value("string"),
                    "words": datasets.Sequence(datasets.Value("string")),
                    "segments": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(datasets.Value("string")),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            homepage="https://alt.qcri.org/resources/da_resources/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = {dialect: _URL + f"seg_plus_pos_{dialect}.txt" for dialect in _DIALECTS}
        dl_dir = dl_manager.download_and_extract(urls_to_download)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": dl_dir[self.config.dialect]},
            )
        ]

    def _generate_examples(self, filepath):
        """Yields examples in the raw (text) form."""
        with open(filepath, encoding="utf-8") as csv_file:
            reader = csv.DictReader(csv_file, delimiter="\t", quoting=csv.QUOTE_NONE)
            fold = -1
            subfold = ""
            words = []
            segments = []
            pos_tags = []
            curr_sent = -1
            for idx, row in enumerate(reader):
                # first example
                if fold == -1:
                    fold = row["Fold"]
                    subfold = row["SubFold"]
                    curr_sent = int(row["SentID"])
                if int(row["SentID"]) != curr_sent:
                    yield curr_sent, {
                        "fold": fold,
                        "subfold": subfold,
                        "words": words,
                        "segments": segments,
                        "pos_tags": pos_tags,
                    }
                    fold = row["Fold"]
                    subfold = row["SubFold"]
                    words = [row["Word"]]
                    segments = [row["Segmentation"]]
                    pos_tags = [row["POS"]]
                    curr_sent = int(row["SentID"])
                else:
                    words.append(row["Word"])
                    segments.append(row["Segmentation"])
                    pos_tags.append(row["POS"])
            # last example
            yield curr_sent, {
                "fold": fold,
                "subfold": subfold,
                "words": words,
                "segments": segments,
                "pos_tags": pos_tags,
            }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""DART: Open-Domain Structured Data Record to Text Generation"""

import json

import datasets


_CITATION = """\
@article{radev2020dart,
  title={DART: Open-Domain Structured Data Record to Text Generation},
  author={Dragomir Radev and Rui Zhang and Amrit Rau and Abhinand Sivaprasad and Chiachun Hsieh and Nazneen Fatema Rajani and Xiangru Tang and Aadit Vyas and Neha Verma and Pranav Krishna and Yangxiaokang Liu and Nadia Irwanto and Jessica Pan and Faiaz Rahman and Ahmad Zaidi and Murori Mutuma and Yasin Tarabar and Ankit Gupta and Tao Yu and Yi Chern Tan and Xi Victoria Lin and Caiming Xiong and Richard Socher},
  journal={arXiv preprint arXiv:2007.02871},
  year={2020}
"""

_DESCRIPTION = """\
DART is a large and open-domain structured DAta Record to Text generation corpus with high-quality
sentence annotations with each input being a set of entity-relation triples following a tree-structured ontology.
It consists of 82191 examples across different domains with each input being a semantic RDF triple set derived
from data records in tables and the tree ontology of table schema, annotated with sentence description that
covers all facts in the triple set.

DART is released in the following paper where you can find more details and baseline results:
https://arxiv.org/abs/2007.02871
"""

_URL = "https://raw.githubusercontent.com/Yale-LILY/dart/master/data/v1.1.1/"
_TRAINING_FILE = "dart-v1.1.1-full-train.json"
_DEV_FILE = "dart-v1.1.1-full-dev.json"
_TEST_FILE = "dart-v1.1.1-full-test.json"


class Dart(datasets.GeneratorBasedBuilder):
    """Dart dataset."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "tripleset": datasets.Sequence(datasets.Sequence(datasets.Value("string"))),
                    "subtree_was_extended": datasets.Value("bool"),
                    "annotations": datasets.Sequence(
                        {
                            "source": datasets.Value("string"),
                            "text": datasets.Value("string"),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/Yale-LILY/dart",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls_to_download = {
            "train": f"{_URL}{_TRAINING_FILE}",
            "dev": f"{_URL}{_DEV_FILE}",
            "test": f"{_URL}{_TEST_FILE}",
        }
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            data = json.loads(f.read())
            for example_idx, example in enumerate(data):
                yield example_idx, {
                    "tripleset": example["tripleset"],
                    "subtree_was_extended": example.get("subtree_was_extended", None),  # some are missing
                    "annotations": {
                        "source": [annotation["source"] for annotation in example["annotations"]],
                        "text": [annotation["text"] for annotation in example["annotations"]],
                    },
                }



"""TODO(discofuse): Add a description here."""


import csv
import os

import datasets


_URL_ = "https://storage.googleapis.com/gresearch/discofuse/"
_CITATION = """\
@InProceedings{GevaEtAl2019,
  title = {DiscoFuse: A Large-Scale Dataset for Discourse-Based Sentence Fusion},
  author = {Geva, Mor and Malmi, Eric and Szpektor, Idan and Berant, Jonathan},
  booktitle = {Proceedings of the 2019 Annual Conference of the North American Chapter of the Association for Computational Linguistics},
  note = {arXiv preprint arXiv:1902.10526},
  year = {2019}
}

"""

# TODO(discofuse):
_DESCRIPTION = """\
 DISCOFUSE is a large scale dataset for discourse-based sentence fusion.
"""


class DiscofuseConfig(datasets.BuilderConfig):

    """BuilderConfig for Discofuse"""

    def __init__(self, data_url, balanced=False, **kwargs):
        """

        Args:
            balanced: to specify if we want to load the balanced file or the full file
            **kwargs: keyword arguments forwarded to super.
        """
        super(DiscofuseConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.balanced = balanced
        self.data_url = data_url


class Discofuse(datasets.GeneratorBasedBuilder):
    """TODO(discofuse): Short description of my dataset."""

    # TODO(discofuse): Set up version.
    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        DiscofuseConfig(
            name="discofuse-sport", description="sentence fusion", data_url=_URL_ + "discofuse_v1_sports.zip"
        ),
        DiscofuseConfig(
            name="discofuse-wikipedia", description="sentence fusion", data_url=_URL_ + "discofuse_v1_wikipedia.zip"
        ),
    ]

    def _info(self):
        # TODO(discofuse): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "connective_string": datasets.Value("string"),
                    "discourse_type": datasets.Value("string"),
                    "coherent_second_sentence": datasets.Value("string"),
                    "has_coref_type_pronoun": datasets.Value("float32"),
                    "incoherent_first_sentence": datasets.Value("string"),
                    "incoherent_second_sentence": datasets.Value("string"),
                    "has_coref_type_nominal": datasets.Value("float32"),
                    "coherent_first_sentence": datasets.Value("string"),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/google-research-datasets/discofuse",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(discofuse): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        if self.config.name == "discofuse-sport":
            dl_dir = dl_manager.download_and_extract(self.config.data_url)
            data_dir = os.path.join(dl_dir, "discofuse_v1/sports")
            if self.config.balanced:
                return [
                    datasets.SplitGenerator(
                        name=datasets.Split.TRAIN,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "train_balanced.tsv")},
                    ),
                    datasets.SplitGenerator(
                        name=datasets.Split.TEST,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "test_balanced.tsv")},
                    ),
                    datasets.SplitGenerator(
                        name=datasets.Split.VALIDATION,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "dev_balanced.tsv")},
                    ),
                ]
            else:
                return [
                    datasets.SplitGenerator(
                        name=datasets.Split.TRAIN,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "train.tsv")},
                    ),
                    datasets.SplitGenerator(
                        name=datasets.Split.TEST,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "test.tsv")},
                    ),
                    datasets.SplitGenerator(
                        name=datasets.Split.VALIDATION,
                        # These kwargs will be passed to _generate_examples
                        gen_kwargs={"filepath": os.path.join(data_dir, "dev.tsv")},
                    ),
                ]
        else:
            if self.config.name == "discofuse-wikipedia":
                dl_dir = dl_manager.download_and_extract(self.config.data_url)
                data_dir = os.path.join(dl_dir, "discofuse_v1/wikipedia")
                if self.config.balanced:
                    return [
                        datasets.SplitGenerator(
                            name=datasets.Split.TRAIN,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "train_balanced.tsv")},
                        ),
                        datasets.SplitGenerator(
                            name=datasets.Split.TEST,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "test_balanced.tsv")},
                        ),
                        datasets.SplitGenerator(
                            name=datasets.Split.VALIDATION,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "dev_balanced.tsv")},
                        ),
                    ]
                else:
                    return [
                        datasets.SplitGenerator(
                            name=datasets.Split.TRAIN,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "train.tsv")},
                        ),
                        datasets.SplitGenerator(
                            name=datasets.Split.TEST,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "test.tsv")},
                        ),
                        datasets.SplitGenerator(
                            name=datasets.Split.VALIDATION,
                            # These kwargs will be passed to _generate_examples
                            gen_kwargs={"filepath": os.path.join(data_dir, "dev.tsv")},
                        ),
                    ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(discofuse): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = csv.DictReader(f, delimiter="\t")
            for id_, row in enumerate(data):
                co_first_sent = row["coherent_first_sentence"]
                co_second_sent = row["coherent_second_sentence"]
                connect_str = row["connective_string"]
                discourse_type = row["discourse_type"]
                has_coref_pronoun = row["has_coref_type_pronoun"]
                has_coref_nominal = row["has_coref_type_nominal"]
                inco_first_sent = row["incoherent_first_sentence"]
                inco_second_sent = row["incoherent_second_sentence"]
                yield id_, {
                    "connective_string": connect_str,
                    "discourse_type": discourse_type,
                    "coherent_second_sentence": co_second_sent,
                    "has_coref_type_pronoun": has_coref_pronoun,
                    "incoherent_first_sentence": inco_first_sent,
                    "incoherent_second_sentence": inco_second_sent,
                    "has_coref_type_nominal": has_coref_nominal,
                    "coherent_first_sentence": co_first_sent,
                }



"""TODO(wikitext): Add a description here."""


import os

import datasets


# TODO(wikitext): BibTeX citation
_CITATION = """\
@article{cruz2019evaluating,
  title={Evaluating Language Model Finetuning Techniques for Low-resource Languages},
  author={Cruz, Jan Christian Blaise and Cheng, Charibeth},
  journal={arXiv preprint arXiv:1907.00409},
  year={2019}
}
"""

# TODO(wikitext):
_DESCRIPTION = """\
Large scale, unlabeled text dataset with 39 Million tokens in the training set. Inspired by the original WikiText Long Term Dependency dataset (Merity et al., 2016). TL means "Tagalog." Originally published in Cruz & Cheng (2019).
"""
_URL = "https://github.com/jcblaisecruz02/Filipino-Text-Benchmarks"
_LICENSE = "GPL-3.0"
_DATA_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/wikitext-tl-39"


class WikitextTl39Config(datasets.BuilderConfig):
    """BuilderConfig for WikiText."""

    def __init__(self, data_url, **kwargs):
        """BuilderConfig for Wikitext

        Args:
          data_url: `string`, url to the dataset (word or raw level)
          **kwargs: keyword arguments forwarded to super.
        """
        super(WikitextTl39Config, self).__init__(
            version=datasets.Version(
                "1.0.0",
            ),
            **kwargs,
        )
        self.data_url = data_url


class WikitextTl39(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        WikitextTl39Config(
            name="wikitext-tl-39",
            data_url=_DATA_URL + "/" + "wikitext-tl-39.zip",
            description=_DESCRIPTION,
        ),
    ]
    BUILDER_CONFIG_CLASS = WikitextTl39Config

    def _info(self):
        # TODO(wikitext): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "text": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_URL,
            citation=_CITATION,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(wikitext): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        if self.config.name == "wikitext-tl-39":
            data_file = dl_manager.download_and_extract(self.config.data_url)
            data_dir = os.path.join(data_file, "wikitext-tl-39")
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"data_file": os.path.join(data_dir, "test.txt"), "split": "test"},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"data_file": os.path.join(data_dir, "train.txt"), "split": "train"},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"data_file": os.path.join(data_dir, "valid.txt"), "split": "valid"},
                ),
            ]

    def _generate_examples(self, data_file, split):

        """Yields examples."""
        # TODO(wikitext): Yields (key, example) tuples from the dataset
        with open(data_file, encoding="utf-8") as f:
            for idx, row in enumerate(f):
                if row.strip():
                    yield idx, {"text": row}
                else:
                    yield idx, {"text": ""}



"""ARCD: Arabic Reading Comprehension Dataset."""


import json

import datasets
from datasets.tasks import QuestionAnsweringExtractive


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{mozannar-etal-2019-neural,
    title = {Neural {A}rabic Question Answering},
    author = {Mozannar, Hussein  and Maamary, Elie  and El Hajal, Karl  and Hajj, Hazem},
    booktitle = {Proceedings of the Fourth Arabic Natural Language Processing Workshop},
    month = {aug},
    year = {2019},
    address = {Florence, Italy},
    publisher = {Association for Computational Linguistics},
    url = {https://www.aclweb.org/anthology/W19-4612},
    doi = {10.18653/v1/W19-4612},
    pages = {108--118},
    abstract = {This paper tackles the problem of open domain factual Arabic question answering (QA) using Wikipedia as our knowledge source. This constrains the answer of any question to be a span of text in Wikipedia. Open domain QA for Arabic entails three challenges: annotated QA datasets in Arabic, large scale efficient information retrieval and machine reading comprehension. To deal with the lack of Arabic QA datasets we present the Arabic Reading Comprehension Dataset (ARCD) composed of 1,395 questions posed by crowdworkers on Wikipedia articles, and a machine translation of the Stanford Question Answering Dataset (Arabic-SQuAD). Our system for open domain question answering in Arabic (SOQAL) is based on two components: (1) a document retriever using a hierarchical TF-IDF approach and (2) a neural reading comprehension model using the pre-trained bi-directional transformer BERT. Our experiments on ARCD indicate the effectiveness of our approach with our BERT-based reader achieving a 61.3 F1 score, and our open domain system SOQAL achieving a 27.6 F1 score.}
}
"""

_DESCRIPTION = """\
 Arabic Reading Comprehension Dataset (ARCD) composed of 1,395 questions\
      posed by crowdworkers on Wikipedia articles.
"""

_URL = "https://raw.githubusercontent.com/husseinmozannar/SOQAL/master/data/"
_URLs = {
    "train": _URL + "arcd-train.json",
    "dev": _URL + "arcd-test.json",
}


class ArcdConfig(datasets.BuilderConfig):
    """BuilderConfig for ARCD."""

    def __init__(self, **kwargs):
        """BuilderConfig for ARCD.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(ArcdConfig, self).__init__(**kwargs)


class Arcd(datasets.GeneratorBasedBuilder):
    """ARCD: Arabic Reading Comprehension Dataset."""

    BUILDER_CONFIGS = [
        ArcdConfig(
            name="plain_text",
            version=datasets.Version("1.0.0", ""),
            description="Plain text",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {"text": datasets.Value("string"), "answer_start": datasets.Value("int32")}
                    ),
                }
            ),
            # No default supervised_keys (as we have to pass both question
            # and context as input).
            supervised_keys=None,
            homepage="https://github.com/husseinmozannar/SOQAL/tree/master/data",
            citation=_CITATION,
            task_templates=[
                QuestionAnsweringExtractive(
                    question_column="question", context_column="context", answers_column="answers"
                )
            ],
        )

    def _split_generators(self, dl_manager):
        urls_to_download = _URLs
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
        ]

    def _generate_examples(self, filepath):
        """This function returns the examples in the raw (text) form."""
        logger.info("generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            arcd = json.load(f)
            for article in arcd["data"]:
                title = article.get("title", "").strip()
                for paragraph in article["paragraphs"]:
                    context = paragraph["context"].strip()
                    for qa in paragraph["qas"]:
                        question = qa["question"].strip()
                        id_ = qa["id"]

                        answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                        answers = [answer["text"].strip() for answer in qa["answers"]]

                        # Features currently used are "context", "question", and "answers".
                        # Others are extracted here for the ease of future expansions.
                        yield id_, {
                            "title": title,
                            "context": context,
                            "question": question,
                            "id": id_,
                            "answers": {"answer_start": answer_starts, "text": answers},
                        }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""BigPatent Dataset."""
import glob
import gzip
import json
import os

import datasets


_CITATION = """
@misc{sharma2019bigpatent,
    title={BIGPATENT: A Large-Scale Dataset for Abstractive and Coherent Summarization},
    author={Eva Sharma and Chen Li and Lu Wang},
    year={2019},
    eprint={1906.03741},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """
BIGPATENT, consisting of 1.3 million records of U.S. patent documents
along with human written abstractive summaries.
Each US patent application is filed under a Cooperative Patent Classification
(CPC) code. There are nine such classification categories:
A (Human Necessities), B (Performing Operations; Transporting),
C (Chemistry; Metallurgy), D (Textiles; Paper), E (Fixed Constructions),
F (Mechanical Engineering; Lightning; Heating; Weapons; Blasting),
G (Physics), H (Electricity), and
Y (General tagging of new or cross-sectional technology)
There are two features:
  - description: detailed description of patent.
  - abstract: Patent abastract.
"""

_URL = "https://drive.google.com/uc?export=download&id=1J3mucMFTWrgAYa3LuBZoLRR3CzzYD3fa"

_DOCUMENT = "description"
_SUMMARY = "abstract"

_CPC_DESCRIPTION = {
    "a": "Human Necessities",
    "b": "Performing Operations; Transporting",
    "c": "Chemistry; Metallurgy",
    "d": "Textiles; Paper",
    "e": "Fixed Constructions",
    "f": "Mechanical Engineering; Lightning; Heating; Weapons; Blasting",
    "g": "Physics",
    "h": "Electricity",
    "y": "General tagging of new or cross-sectional technology",
}

# Available versions:
# 1.0.0 lower cased tokenized words.
# 2.0.0 cased raw strings.
# 2.1.0 cased raw strings (fixed).
# TODO Add raw string versions

_VERSION = "1.0.0"


class BigPatentConfig(datasets.BuilderConfig):
    """BuilderConfig for BigPatent."""

    def __init__(self, *args, cpc_codes=None, **kwargs):
        """BuilderConfig for BigPatent.
        Args:
            cpc_codes: str, cpc_codes
            **kwargs: keyword arguments forwarded to super.
        """
        super().__init__(*args, version=_VERSION, **kwargs)
        self.cpc_codes = cpc_codes


class BigPatent(datasets.GeneratorBasedBuilder):
    """BigPatent datasets."""

    BUILDER_CONFIGS = [
        BigPatentConfig(
            cpc_codes=list(_CPC_DESCRIPTION),
            name="all",
            description="Patents under all categories.",
        ),
    ] + [
        BigPatentConfig(  # pylint:disable=g-complex-comprehension
            cpc_codes=[k],
            name=k,
            description=("Patents under Cooperative Patent Classification (CPC)" f"{k}: {v}"),
        )
        for k, v in sorted(_CPC_DESCRIPTION.items())
    ]
    DEFAULT_CONFIG_NAME = "all"
    VERSION = _VERSION

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features({_DOCUMENT: datasets.Value("string"), _SUMMARY: datasets.Value("string")}),
            supervised_keys=(_DOCUMENT, _SUMMARY),
            homepage="https://evasharma.github.io/bigpatent/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        dl_path = dl_manager.download_and_extract(_URL)
        split_types = ["train", "val", "test"]
        extract_paths = dl_manager.extract(
            {k: os.path.join(dl_path, "bigPatentData", k + ".tar.gz") for k in split_types}
        )
        extract_paths = {k: os.path.join(extract_paths[k], k) for k in split_types}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"path": extract_paths["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"path": extract_paths["val"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"path": extract_paths["test"]},
            ),
        ]

    def _generate_examples(self, path=None):
        """Yields examples."""
        for cpc_code in self.config.cpc_codes:
            filenames = glob.glob(os.path.join(path, cpc_code, "*"))
            for filename in sorted(filenames):
                with open(filename, "rb") as fin:
                    fin = gzip.GzipFile(fileobj=fin)
                    for row in fin:
                        json_obj = json.loads(row)
                        yield json_obj["publication_number"], {
                            _DOCUMENT: json_obj[_DOCUMENT],
                            _SUMMARY: json_obj[_SUMMARY],
                        }



import warnings
from functools import wraps
from typing import Callable, Optional

from .logging import get_logger


_emitted_deprecation_warnings = set()
logger = get_logger(__name__)


def deprecated(help_message: Optional[str] = None):
    """Decorator to mark a function as deprecated.

    Args:
        help_message (`Optional[str]`): An optional message to guide the user on how to
            switch to non-deprecated usage of the library.
    """

    def decorator(deprecated_function: Callable):
        global _emitted_deprecation_warnings
        warning_msg = (
            (
                f"{deprecated_function.__name__} is deprecated and will be removed "
                "in the next major version of datasets."
            )
            + f" {help_message}"
            if help_message
            else ""
        )

        @wraps(deprecated_function)
        def wrapper(*args, **kwargs):
            func_hash = hash(deprecated_function)
            if func_hash not in _emitted_deprecation_warnings:
                warnings.warn(warning_msg, category=FutureWarning, stacklevel=2)
                _emitted_deprecation_warnings.add(func_hash)
            return deprecated_function(*args, **kwargs)

        wrapper._decorator_name_ = "deprecated"
        return wrapper

    return decorator



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Yahoo Non-Factoid Question Dataset"""


import json
import os

import datasets


logger = datasets.logging.get_logger(__name__)


_DESCRIPTION = """\
Yahoo Non-Factoid Question Dataset is derived from Yahoo's Webscope L6 collection using machine learning techiques such \
that the questions would contain non-factoid answers.The dataset contains 87,361 questions and their corresponding answers. \
Each question contains its best answer along with additional other answers submitted by users. \
Only the best answer was reviewed in determining the quality of the question-answer pair.
"""

_URL = "https://ciir.cs.umass.edu/downloads/nfL6/nfL6.json.gz"


class YahooAnswersQa(datasets.GeneratorBasedBuilder):
    """Yahoo Non-Factoid Question Dataset"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [datasets.BuilderConfig(name="yahoo_answers_qa", version=datasets.Version("1.0.0"))]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "nbestanswers": datasets.features.Sequence(datasets.Value("string")),
                    "main_category": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://ciir.cs.umass.edu/downloads/nfL6/index.html",
        )

    def _split_generators(self, dl_manager):
        downloaded_file = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_file}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        if os.path.isdir(filepath):
            filepath = os.path.join(filepath, "nfL6.json")
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for example in data:
                yield example["id"], example



from dataclasses import dataclass
from typing import Optional

import pyarrow as pa

import datasets


logger = datasets.utils.logging.get_logger(__name__)


@dataclass
class TextConfig(datasets.BuilderConfig):
    """BuilderConfig for text files."""

    features: Optional[datasets.Features] = None
    encoding: str = "utf-8"
    chunksize: int = 10 << 20  # 10MB
    keep_linebreaks: bool = False
    sample_by: str = "line"


class Text(datasets.ArrowBasedBuilder):
    BUILDER_CONFIG_CLASS = TextConfig

    def _info(self):
        return datasets.DatasetInfo(features=self.config.features)

    def _split_generators(self, dl_manager):
        """The `data_files` kwarg in load_dataset() can be a str, List[str], Dict[str,str], or Dict[str,List[str]].

        If str or List[str], then the dataset returns only the 'train' split.
        If dict, then keys should be from the `datasets.Split` enum.
        """
        if not self.config.data_files:
            raise ValueError(f"At least one data file must be specified, but got data_files={self.config.data_files}")
        data_files = dl_manager.download_and_extract(self.config.data_files)
        if isinstance(data_files, (str, list, tuple)):
            files = data_files
            if isinstance(files, str):
                files = [files]
            return [
                datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"files": dl_manager.iter_files(files)})
            ]
        splits = []
        for split_name, files in data_files.items():
            if isinstance(files, str):
                files = [files]
            splits.append(datasets.SplitGenerator(name=split_name, gen_kwargs={"files": dl_manager.iter_files(files)}))
        return splits

    def _generate_tables(self, files):
        schema = pa.schema(self.config.features.type if self.config.features is not None else {"text": pa.string()})
        for file_idx, file in enumerate(files):
            batch_idx = 0
            with open(file, "r", encoding=self.config.encoding) as f:
                if self.config.sample_by == "line":
                    batch_idx = 0
                    while True:
                        batch = f.read(self.config.chunksize)
                        if not batch:
                            break
                        batch += f.readline()  # finish current line
                        batch = batch.splitlines(keepends=self.config.keep_linebreaks)
                        pa_table = pa.Table.from_arrays([pa.array(batch)], schema=schema)
                        # Uncomment for debugging (will print the Arrow table size and elements)
                        # logger.warning(f"pa_table: {pa_table} num rows: {pa_table.num_rows}")
                        # logger.warning('\n'.join(str(pa_table.slice(i, 1).to_pydict()) for i in range(pa_table.num_rows)))
                        yield (file_idx, batch_idx), pa_table
                        batch_idx += 1
                elif self.config.sample_by == "paragraph":
                    batch_idx = 0
                    batch = ""
                    while True:
                        batch += f.read(self.config.chunksize)
                        if not batch:
                            break
                        batch += f.readline()  # finish current line
                        batch = batch.split("\n\n")
                        pa_table = pa.Table.from_arrays(
                            [pa.array([example for example in batch[:-1] if example])], schema=schema
                        )
                        # Uncomment for debugging (will print the Arrow table size and elements)
                        # logger.warning(f"pa_table: {pa_table} num rows: {pa_table.num_rows}")
                        # logger.warning('\n'.join(str(pa_table.slice(i, 1).to_pydict()) for i in range(pa_table.num_rows)))
                        yield (file_idx, batch_idx), pa_table
                        batch_idx += 1
                        batch = batch[-1]
                elif self.config.sample_by == "document":
                    text = f.read()
                    pa_table = pa.Table.from_arrays([pa.array([text])], schema=schema)
                    yield file_idx, pa_table



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The IndoNLU benchmark is a collection of resources for training, evaluating, and analyzing natural language understanding systems for Bahasa Indonesia"""


import ast
import csv
import textwrap

import datasets


_INDONLU_CITATION = """\
@inproceedings{wilie2020indonlu,
title = {{IndoNLU}: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
authors={Bryan Wilie and Karissa Vincentio and Genta Indra Winata and Samuel Cahyawijaya and X. Li and Zhi Yuan Lim and S. Soleman and R. Mahendra and Pascale Fung and Syafri Bahar and A. Purwarianti},
booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
year={2020}
}
"""

_INDONLU_DESCRIPTION = """\
The IndoNLU benchmark is a collection of resources for training, evaluating, \
and analyzing natural language understanding systems for Bahasa Indonesia.
"""

_INDONLU_HOMEPAGE = "https://www.indobenchmark.com/"

_INDONLU_LICENSE = "https://raw.githubusercontent.com/indobenchmark/indonlu/master/LICENSE"


class IndonluConfig(datasets.BuilderConfig):
    """BuilderConfig for IndoNLU"""

    def __init__(
        self,
        text_features,
        label_column,
        label_classes,
        train_url,
        valid_url,
        test_url,
        citation,
        **kwargs,
    ):
        """BuilderConfig for IndoNLU.

        Args:
          text_features: `dict[string, string]`, map from the name of the feature
            dict for each text field to the name of the column in the txt/csv/tsv file
          label_column: `string`, name of the column in the txt/csv/tsv file corresponding
            to the label
          label_classes: `list[string]`, the list of classes if the label is categorical
          train_url: `string`, url to train file from
          valid_url: `string`, url to valid file from
          test_url: `string`, url to test file from
          citation: `string`, citation for the data set
          **kwargs: keyword arguments forwarded to super.
        """
        super(IndonluConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.text_features = text_features
        self.label_column = label_column
        self.label_classes = label_classes
        self.train_url = train_url
        self.valid_url = valid_url
        self.test_url = test_url
        self.citation = citation


class Indonlu(datasets.GeneratorBasedBuilder):
    """Indonesian Natural Language Understanding (IndoNLU) benchmark"""

    BUILDER_CONFIGS = [
        IndonluConfig(
            name="emot",
            description=textwrap.dedent(
                """\
            An emotion classification dataset collected from the social media
            platform Twitter (Saputri et al., 2018). The dataset consists of
            around 4000 Indonesian colloquial language tweets, covering five
            different emotion labels: sadness, anger, love, fear, and happy."""
            ),
            text_features={"tweet": "tweet"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["sadness", "anger", "love", "fear", "happy"],
            label_column="label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/emot_emotion-twitter/train_preprocess.csv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/emot_emotion-twitter/valid_preprocess.csv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/emot_emotion-twitter/test_preprocess_masked_label.csv",
            citation=textwrap.dedent(
                """\
            @inproceedings{saputri2018emotion,
              title={Emotion Classification on Indonesian Twitter Dataset},
              author={Mei Silviana Saputri, Rahmad Mahendra, and Mirna Adriani},
              booktitle={Proceedings of the 2018 International Conference on Asian Language Processing(IALP)},
              pages={90--95},
              year={2018},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="smsa",
            description=textwrap.dedent(
                """\
            This sentence-level sentiment analysis dataset (Purwarianti and Crisdayanti, 2019)
            is a collection of comments and reviews in Indonesian obtained from multiple online
            platforms. The text was crawled and then annotated by several Indonesian linguists
            to construct this dataset. There are three possible sentiments on the SmSA
            dataset: positive, negative, and neutral."""
            ),
            text_features={"text": "text"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["positive", "neutral", "negative"],
            label_column="label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/smsa_doc-sentiment-prosa/train_preprocess.tsv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/smsa_doc-sentiment-prosa/valid_preprocess.tsv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/smsa_doc-sentiment-prosa/test_preprocess_masked_label.tsv",
            citation=textwrap.dedent(
                """\
            @inproceedings{purwarianti2019improving,
              title={Improving Bi-LSTM Performance for Indonesian Sentiment Analysis Using Paragraph Vector},
              author={Ayu Purwarianti and Ida Ayu Putu Ari Crisdayanti},
              booktitle={Proceedings of the 2019 International Conference of Advanced Informatics: Concepts, Theory and Applications (ICAICTA)},
              pages={1--5},
              year={2019},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="casa",
            description=textwrap.dedent(
                """\
            An aspect-based sentiment analysis dataset consisting of around a thousand car reviews collected
            from multiple Indonesian online automobile platforms (Ilmania et al., 2018). The dataset covers
            six aspects of car quality. We define the task to be a multi-label classification task, where
            each label represents a sentiment for a single aspect with three possible values: positive,
            negative, and neutral."""
            ),
            text_features={"sentence": "sentence"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["negative", "neutral", "positive"],
            label_column=["fuel", "machine", "others", "part", "price", "service"],
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/casa_absa-prosa/train_preprocess.csv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/casa_absa-prosa/valid_preprocess.csv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/casa_absa-prosa/test_preprocess_masked_label.csv",
            citation=textwrap.dedent(
                """\
            @inproceedings{ilmania2018aspect,
              title={Aspect Detection and Sentiment Classification Using Deep Neural Network for Indonesian Aspect-based Sentiment Analysis},
              author={Arfinda Ilmania, Abdurrahman, Samuel Cahyawijaya, Ayu Purwarianti},
              booktitle={Proceedings of the 2018 International Conference on Asian Language Processing(IALP)},
              pages={62--67},
              year={2018},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="hoasa",
            description=textwrap.dedent(
                """\
            An aspect-based sentiment analysis dataset consisting of hotel reviews collected from the hotel
            aggregator platform, AiryRooms (Azhar et al., 2019). The dataset covers ten different aspects of
            hotel quality. Each review is labeled with a single sentiment label for each aspect. There are
            four possible sentiment classes for each sentiment label: positive, negative, neutral, and
            positive-negative. The positivenegative label is given to a review that contains multiple sentiments
            of the same aspect but for different objects (e.g., cleanliness of bed and toilet)."""
            ),
            text_features={"sentence": "sentence"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["neg", "neut", "pos", "neg_pos"],
            label_column=[
                "ac",
                "air_panas",
                "bau",
                "general",
                "kebersihan",
                "linen",
                "service",
                "sunrise_meal",
                "tv",
                "wifi",
            ],
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/hoasa_absa-airy/train_preprocess.csv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/hoasa_absa-airy/valid_preprocess.csv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/hoasa_absa-airy/test_preprocess_masked_label.csv",
            citation=textwrap.dedent(
                """\
            @inproceedings{azhar2019multi,
              title={Multi-label Aspect Categorization with Convolutional Neural Networks and Extreme Gradient Boosting},
              author={A. N. Azhar, M. L. Khodra, and A. P. Sutiono}
              booktitle={Proceedings of the 2019 International Conference on Electrical Engineering and Informatics (ICEEI)},
              pages={35--40},
              year={2019}
            }"""
            ),
        ),
        IndonluConfig(
            name="wrete",
            description=textwrap.dedent(
                """\
            The Wiki Revision Edits Textual Entailment dataset (Setya and Mahendra, 2018) consists of 450 sentence pairs
            constructed from Wikipedia revision history. The dataset contains pairs of sentences and binary semantic
            relations between the pairs. The data are labeled as entailed when the meaning of the second sentence can be
            derived from the first one, and not entailed otherwise."""
            ),
            text_features={
                "premise": "premise",
                "hypothesis": "hypothesis",
                "category": "category",
            },
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["NotEntail", "Entail_or_Paraphrase"],
            label_column="label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/wrete_entailment-ui/train_preprocess.csv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/wrete_entailment-ui/valid_preprocess.csv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/wrete_entailment-ui/test_preprocess_masked_label.csv",
            citation=textwrap.dedent(
                """\
            @inproceedings{setya2018semi,
              title={Semi-supervised Textual Entailment on Indonesian Wikipedia Data},
              author={Ken Nabila Setya and Rahmad Mahendra},
              booktitle={Proceedings of the 2018 International Conference on Computational Linguistics and Intelligent Text Processing (CICLing)},
              year={2018}
            }"""
            ),
        ),
        IndonluConfig(
            name="posp",
            description=textwrap.dedent(
                """\
            This Indonesian part-of-speech tagging (POS) dataset (Hoesen and Purwarianti, 2018) is collected from Indonesian
            news websites. The dataset consists of around 8000 sentences with 26 POS tags. The POS tag labels follow the
            Indonesian Association of Computational Linguistics (INACL) POS Tagging Convention."""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=[
                "B-PPO",
                "B-KUA",
                "B-ADV",
                "B-PRN",
                "B-VBI",
                "B-PAR",
                "B-VBP",
                "B-NNP",
                "B-UNS",
                "B-VBT",
                "B-VBL",
                "B-NNO",
                "B-ADJ",
                "B-PRR",
                "B-PRK",
                "B-CCN",
                "B-$$$",
                "B-ADK",
                "B-ART",
                "B-CSN",
                "B-NUM",
                "B-SYM",
                "B-INT",
                "B-NEG",
                "B-PRI",
                "B-VBE",
            ],
            label_column="pos_tags",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/posp_pos-prosa/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/posp_pos-prosa/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/posp_pos-prosa/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @inproceedings{hoesen2018investigating,
              title={Investigating Bi-LSTM and CRF with POS Tag Embedding for Indonesian Named Entity Tagger},
              author={Devin Hoesen and Ayu Purwarianti},
              booktitle={Proceedings of the 2018 International Conference on Asian Language Processing (IALP)},
              pages={35--38},
              year={2018},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="bapos",
            description=textwrap.dedent(
                """\
            This POS tagging dataset (Dinakaramani et al., 2014) contains about 1000 sentences, collected from the PAN Localization
            Project. In this dataset, each word is tagged by one of 23 POS tag classes. Data splitting used in this benchmark follows
            the experimental setting used by Kurniawan and Aji (2018)"""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=[
                "B-PR",
                "B-CD",
                "I-PR",
                "B-SYM",
                "B-JJ",
                "B-DT",
                "I-UH",
                "I-NND",
                "B-SC",
                "I-WH",
                "I-IN",
                "I-NNP",
                "I-VB",
                "B-IN",
                "B-NND",
                "I-CD",
                "I-JJ",
                "I-X",
                "B-OD",
                "B-RP",
                "B-RB",
                "B-NNP",
                "I-RB",
                "I-Z",
                "B-CC",
                "B-NEG",
                "B-VB",
                "B-NN",
                "B-MD",
                "B-UH",
                "I-NN",
                "B-PRP",
                "I-SC",
                "B-Z",
                "I-PRP",
                "I-OD",
                "I-SYM",
                "B-WH",
                "B-FW",
                "I-CC",
                "B-X",
            ],
            label_column="pos_tags",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/bapos_pos-idn/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/bapos_pos-idn/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/bapos_pos-idn/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @inproceedings{dinakaramani2014designing,
              title={Designing an Indonesian Part of Speech Tagset and Manually Tagged Indonesian Corpus},
              author={Arawinda Dinakaramani, Fam Rashel, Andry Luthfi, and Ruli Manurung},
              booktitle={Proceedings of the 2014 International Conference on Asian Language Processing (IALP)},
              pages={66--69},
              year={2014},
              organization={IEEE}
            }
            @inproceedings{kurniawan2019toward,
              title={Toward a Standardized and More Accurate Indonesian Part-of-Speech Tagging},
              author={Kemal Kurniawan and Alham Fikri Aji},
              booktitle={Proceedings of the 2018 International Conference on Asian Language Processing (IALP)},
              pages={303--307},
              year={2018},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="terma",
            description=textwrap.dedent(
                """\
            This span-extraction dataset is collected from the hotel aggregator platform, AiryRooms (Septiandri and Sutiono, 2019;
            Fernando et al., 2019). The dataset consists of thousands of hotel reviews, which each contain a span label for aspect
            and sentiment words representing the opinion of the reviewer on the corresponding aspect. The labels use
            Inside-Outside-Beginning (IOB) tagging representation with two kinds of tags, aspect and sentiment."""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["I-SENTIMENT", "O", "I-ASPECT", "B-SENTIMENT", "B-ASPECT"],
            label_column="seq_label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/terma_term-extraction-airy/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/terma_term-extraction-airy/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/terma_term-extraction-airy/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @article{winatmoko2019aspect,
              title={Aspect and Opinion Term Extraction for Hotel Reviews Using Transfer Learning and Auxiliary Labels},
              author={Yosef Ardhito Winatmoko, Ali Akbar Septiandri, Arie Pratama Sutiono},
              journal={arXiv preprint arXiv:1909.11879},
              year={2019}
            }
            @article{fernando2019aspect,
              title={Aspect and Opinion Terms Extraction Using Double Embeddings and Attention Mechanism for Indonesian Hotel Reviews},
              author={Jordhy Fernando, Masayu Leylia Khodra, Ali Akbar Septiandri},
              journal={arXiv preprint arXiv:1908.04899},
              year={2019}
            }"""
            ),
        ),
        IndonluConfig(
            name="keps",
            description=textwrap.dedent(
                """\
            This keyphrase extraction dataset (Mahfuzh et al., 2019) consists of text from Twitter discussing
            banking products and services and is written in the Indonesian language. A phrase containing
            important information is considered a keyphrase. Text may contain one or more keyphrases since
            important phrases can be located at different positions. The dataset follows the IOB chunking format,
            which represents the position of the keyphrase."""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["O", "B", "I"],
            label_column="seq_label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/keps_keyword-extraction-prosa/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/keps_keyword-extraction-prosa/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/keps_keyword-extraction-prosa/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @inproceedings{mahfuzh2019improving,
              title={Improving Joint Layer RNN based Keyphrase Extraction by Using Syntactical Features},
              author={Miftahul Mahfuzh, Sidik Soleman, and Ayu Purwarianti},
              booktitle={Proceedings of the 2019 International Conference of Advanced Informatics: Concepts, Theory and Applications (ICAICTA)},
              pages={1--6},
              year={2019},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="nergrit",
            description=textwrap.dedent(
                """\
            This NER dataset is taken from the Grit-ID repository, and the labels are spans in IOB chunking representation.
            The dataset consists of three kinds of named entity tags, PERSON (name of person), PLACE (name of location), and
            ORGANIZATION (name of organization)."""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["I-PERSON", "B-ORGANISATION", "I-ORGANISATION", "B-PLACE", "I-PLACE", "O", "B-PERSON"],
            label_column="ner_tags",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nergrit_ner-grit/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nergrit_ner-grit/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nergrit_ner-grit/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @online{nergrit2019,
              title={NERGrit Corpus},
              author={NERGrit Developers},
              year={2019},
              url={https://github.com/grit-id/nergrit-corpus}
            }"""
            ),
        ),
        IndonluConfig(
            name="nerp",
            description=textwrap.dedent(
                """\
            This NER dataset (Hoesen and Purwarianti, 2018) contains texts collected from several Indonesian news websites.
            There are five labels available in this dataset, PER (name of person), LOC (name of location), IND (name of product or brand),
            EVT (name of the event), and FNB (name of food and beverage). The NERP dataset uses the IOB chunking format."""
            ),
            text_features={"tokens": "tokens"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=[
                "I-PPL",
                "B-EVT",
                "B-PLC",
                "I-IND",
                "B-IND",
                "B-FNB",
                "I-EVT",
                "B-PPL",
                "I-PLC",
                "O",
                "I-FNB",
            ],
            label_column="ner_tags",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nerp_ner-prosa/train_preprocess.txt",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nerp_ner-prosa/valid_preprocess.txt",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/nerp_ner-prosa/test_preprocess_masked_label.txt",
            citation=textwrap.dedent(
                """\
            @inproceedings{hoesen2018investigating,
              title={Investigating Bi-LSTM and CRF with POS Tag Embedding for Indonesian Named Entity Tagger},
              author={Devin Hoesen and Ayu Purwarianti},
              booktitle={Proceedings of the 2018 International Conference on Asian Language Processing (IALP)},
              pages={35--38},
              year={2018},
              organization={IEEE}
            }"""
            ),
        ),
        IndonluConfig(
            name="facqa",
            description=textwrap.dedent(
                """\
            The goal of the FacQA dataset is to find the answer to a question from a provided short passage from
            a news article (Purwarianti et al., 2007). Each row in the FacQA dataset consists of a question,
            a short passage, and a label phrase, which can be found inside the corresponding short passage.
            There are six categories of questions: date, location, name, organization, person, and quantitative."""
            ),
            text_features={"question": "question", "passage": "passage"},
            # label classes sorted refer to https://github.com/indobenchmark/indonlu/blob/master/utils/data_utils.py
            label_classes=["O", "B", "I"],
            label_column="seq_label",
            train_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/facqa_qa-factoid-itb/train_preprocess.csv",
            valid_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/facqa_qa-factoid-itb/valid_preprocess.csv",
            test_url="https://raw.githubusercontent.com/indobenchmark/indonlu/master/dataset/facqa_qa-factoid-itb/test_preprocess_masked_label.csv",
            citation=textwrap.dedent(
                """\
            @inproceedings{purwarianti2007machine,
              title={A Machine Learning Approach for Indonesian Question Answering System},
              author={Ayu Purwarianti, Masatoshi Tsuchiya, and Seiichi Nakagawa},
              booktitle={Proceedings of Artificial Intelligence and Applications },
              pages={573--578},
              year={2007}
            }"""
            ),
        ),
    ]

    def _info(self):
        sentence_features = ["terma", "keps", "facqa"]
        ner_ = ["nergrit", "nerp"]
        pos_ = ["posp", "bapos"]

        if self.config.name in (sentence_features + ner_ + pos_):
            features = {
                text_feature: datasets.Sequence(datasets.Value("string"))
                for text_feature in self.config.text_features.keys()
            }
        else:
            features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features}

        if self.config.label_classes:
            if self.config.name in sentence_features:
                features["seq_label"] = datasets.Sequence(
                    datasets.features.ClassLabel(names=self.config.label_classes)
                )
            elif self.config.name in ner_:
                features["ner_tags"] = datasets.Sequence(datasets.features.ClassLabel(names=self.config.label_classes))
            elif self.config.name in pos_:
                features["pos_tags"] = datasets.Sequence(datasets.features.ClassLabel(names=self.config.label_classes))
            elif self.config.name == "casa" or self.config.name == "hoasa":
                for label in self.config.label_column:
                    features[label] = datasets.features.ClassLabel(names=self.config.label_classes)
            else:
                features["label"] = datasets.features.ClassLabel(names=self.config.label_classes)

        return datasets.DatasetInfo(
            description=self.config.description,
            features=datasets.Features(features),
            homepage=_INDONLU_HOMEPAGE,
            citation=self.config.citation + "\n" + _INDONLU_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        train_path = dl_manager.download_and_extract(self.config.train_url)
        valid_path = dl_manager.download_and_extract(self.config.valid_url)
        test_path = dl_manager.download_and_extract(self.config.test_url)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": valid_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        csv_file = ["emot", "wrete", "facqa", "casa", "hoasa"]
        tsv_file = ["smsa"]
        txt_file = ["terma", "keps"]
        txt_file_pos = ["posp", "bapos"]
        txt_file_ner = ["nergrit", "nerp"]

        with open(filepath, encoding="utf-8") as f:

            if self.config.name in csv_file:
                reader = csv.reader(f, delimiter=",", quotechar='"', quoting=csv.QUOTE_ALL)
                next(reader)  # skip first row which is header

                for id_, row in enumerate(reader):
                    if self.config.name == "emot":
                        label, tweet = row
                        yield id_, {"tweet": tweet, "label": label}
                    elif self.config.name == "wrete":
                        premise, hypothesis, category, label = row
                        yield id_, {"premise": premise, "hypothesis": hypothesis, "category": category, "label": label}
                    elif self.config.name == "facqa":
                        question, passage, seq_label = row
                        yield id_, {
                            "question": ast.literal_eval(question),
                            "passage": ast.literal_eval(passage),
                            "seq_label": ast.literal_eval(seq_label),
                        }
                    elif self.config.name == "casa" or self.config.name == "hoasa":
                        sentence, *labels = row
                        sentence = {"sentence": sentence}
                        label = {l: labels[idx] for idx, l in enumerate(self.config.label_column)}
                        yield id_, {**sentence, **label}
            elif self.config.name in tsv_file:
                reader = csv.reader(f, delimiter="\t", quoting=csv.QUOTE_NONE)

                for id_, row in enumerate(reader):
                    if self.config.name == "smsa":
                        text, label = row
                        yield id_, {"text": text, "label": label}
            elif self.config.name in (txt_file + txt_file_pos + txt_file_ner):
                id_ = 0
                tokens = []
                seq_label = []
                for line in f:
                    if len(line.strip()) > 0:
                        token, label = line[:-1].split("\t")
                        tokens.append(token)
                        seq_label.append(label)
                    else:
                        if self.config.name in txt_file:
                            yield id_, {"tokens": tokens, "seq_label": seq_label}
                        elif self.config.name in txt_file_pos:
                            yield id_, {"tokens": tokens, "pos_tags": seq_label}
                        elif self.config.name in txt_file_ner:
                            yield id_, {"tokens": tokens, "ner_tags": seq_label}
                        id_ += 1
                        tokens = []
                        seq_label = []
                # add last example
                if tokens:
                    if self.config.name in txt_file:
                        yield id_, {"tokens": tokens, "seq_label": seq_label}
                    elif self.config.name in txt_file_pos:
                        yield id_, {"tokens": tokens, "pos_tags": seq_label}
                    elif self.config.name in txt_file_ner:
                        yield id_, {"tokens": tokens, "ner_tags": seq_label}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""TriviaQA: A Reading Comprehension Dataset."""


import glob
import json
import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """
@article{2017arXivtriviaqa,
       author = {{Joshi}, Mandar and {Choi}, Eunsol and {Weld},
                 Daniel and {Zettlemoyer}, Luke},
        title = "{triviaqa: A Large Scale Distantly Supervised Challenge Dataset for Reading Comprehension}",
      journal = {arXiv e-prints},
         year = 2017,
          eid = {arXiv:1705.03551},
        pages = {arXiv:1705.03551},
archivePrefix = {arXiv},
       eprint = {1705.03551},
}
"""
_DOWNLOAD_URL_TMPL = "http://nlp.cs.washington.edu/triviaqa/data/triviaqa-{}.tar.gz"
_WEB_EVIDENCE_DIR = "evidence/web"
_WIKI_EVIDENCE_DIR = "evidence/wikipedia"

_DESCRIPTION = """\
TriviaqQA is a reading comprehension dataset containing over 650K
question-answer-evidence triples. TriviaqQA includes 95K question-answer
pairs authored by trivia enthusiasts and independently gathered evidence
documents, six per question on average, that provide high quality distant
supervision for answering the questions.
"""

_RC_DESCRIPTION = """\
Question-answer pairs where all documents for a given question contain the
answer string(s).
"""

_UNFILTERED_DESCRIPTION = """\
110k question-answer pairs for open domain QA where not all documents for a
given question contain the answer string(s). This makes the unfiltered dataset
more appropriate for IR-style QA.
"""

_CONTEXT_ADDENDUM = "Includes context from Wikipedia and search results."


def _web_evidence_dir(tmp_dir):
    return sorted(glob.glob(os.path.join(tmp_dir, _WEB_EVIDENCE_DIR)))


def _wiki_evidence_dir(tmp_dir):
    return sorted(glob.glob(os.path.join(tmp_dir, _WIKI_EVIDENCE_DIR)))


def _qa_files(file_paths, sources, split, unfiltered):
    qa_dir = (
        os.path.join(file_paths["unfiltered"], "triviaqa-unfiltered")
        if unfiltered
        else os.path.join(file_paths["rc"], "qa")
    )

    suffix_mapping = {
        datasets.Split.TRAIN: "train.json",
        datasets.Split.VALIDATION: "dev.json",
        datasets.Split.TEST: "test-without-answers.json",
    }
    suffix = suffix_mapping[split]

    filenames = [f"unfiltered-web-{suffix}"] if unfiltered else [f"{source}-{suffix}" for source in sources]

    filenames = [os.path.join(qa_dir, filename) for filename in filenames]

    return sorted(filenames)


class TriviaQaConfig(datasets.BuilderConfig):
    """BuilderConfig for TriviaQA."""

    def __init__(self, source="all", unfiltered=False, exclude_context=False, **kwargs):
        """BuilderConfig for TriviaQA.

        Args:
          unfiltered: bool, whether to use the unfiltered version of the dataset,
            intended for open-domain QA.
          exclude_context: bool, whether to exclude Wikipedia and search context for
            reduced size.
          **kwargs: keyword arguments forwarded to super.
        """
        name = "unfiltered" if unfiltered else "rc"

        assert source in ["all", "web", "wikipedia"]

        # there is no unfiltered version for the wikipedia subset
        # --> unfiltered subset for source="all" is the same as for source="web"
        # --> only accept source="all" if unfiltered is True
        assert not unfiltered or source == "all"

        if source != "all":
            name += f".{source}"

        if exclude_context:
            name += ".nocontext"
        description = _UNFILTERED_DESCRIPTION if unfiltered else _RC_DESCRIPTION
        if not exclude_context:
            description += _CONTEXT_ADDENDUM
        super(TriviaQaConfig, self).__init__(
            name=name, description=description, version=datasets.Version("1.2.0"), **kwargs
        )

        self.sources = ["web", "wikipedia"] if source == "all" else [source]
        self.unfiltered = unfiltered
        self.exclude_context = exclude_context


class TriviaQa(datasets.GeneratorBasedBuilder):
    """TriviaQA is a reading comprehension dataset.

    It containss over 650K question-answer-evidence triples.
    """

    BUILDER_CONFIGS = [
        TriviaQaConfig(source="all", unfiltered=False, exclude_context=False),  # rc
        TriviaQaConfig(source="all", unfiltered=False, exclude_context=True),  # rc.nocontext
        TriviaQaConfig(source="all", unfiltered=True, exclude_context=False),  # unfiltered
        TriviaQaConfig(source="all", unfiltered=True, exclude_context=True),  # unfilered.nocontext
        TriviaQaConfig(source="web", unfiltered=False, exclude_context=False),  # rc
        TriviaQaConfig(source="web", unfiltered=False, exclude_context=True),  # rc.nocontext
        TriviaQaConfig(source="wikipedia", unfiltered=False, exclude_context=False),  # rc
        TriviaQaConfig(source="wikipedia", unfiltered=False, exclude_context=True),  # rc.nocontext
    ]
    DEFAULT_WRITER_BATCH_SIZE = 1000  # examples are quite big, so set this value to save some RAM

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "question": datasets.Value("string"),
                    "question_id": datasets.Value("string"),
                    "question_source": datasets.Value("string"),
                    "entity_pages": datasets.features.Sequence(
                        {
                            "doc_source": datasets.Value("string"),
                            "filename": datasets.Value("string"),
                            "title": datasets.Value("string"),
                            "wiki_context": datasets.Value("string"),
                        }
                    ),
                    "search_results": datasets.features.Sequence(
                        {
                            "description": datasets.Value("string"),
                            "filename": datasets.Value("string"),
                            "rank": datasets.Value("int32"),
                            "title": datasets.Value("string"),
                            "url": datasets.Value("string"),
                            "search_context": datasets.Value("string"),
                        }
                    ),
                    "answer": dict(
                        {
                            "aliases": datasets.features.Sequence(datasets.Value("string")),
                            "normalized_aliases": datasets.features.Sequence(datasets.Value("string")),
                            "matched_wiki_entity_name": datasets.Value("string"),
                            "normalized_matched_wiki_entity_name": datasets.Value("string"),
                            "normalized_value": datasets.Value("string"),
                            "type": datasets.Value("string"),
                            "value": datasets.Value("string"),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage="http://nlp.cs.washington.edu/triviaqa/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        cfg = self.config
        download_urls = dict()
        if not (cfg.unfiltered and cfg.exclude_context):
            download_urls["rc"] = _DOWNLOAD_URL_TMPL.format("rc")
        if cfg.unfiltered:
            download_urls["unfiltered"] = _DOWNLOAD_URL_TMPL.format("unfiltered")
        file_paths = dl_manager.download_and_extract(download_urls)

        if cfg.exclude_context:
            web_evidence_dir = None
            wiki_evidence_dir = None
        else:
            web_evidence_dir = os.path.join(file_paths["rc"], _WEB_EVIDENCE_DIR)
            wiki_evidence_dir = os.path.join(file_paths["rc"], _WIKI_EVIDENCE_DIR)

        return [
            datasets.SplitGenerator(
                name=name,
                gen_kwargs={
                    "files": _qa_files(file_paths, cfg.sources, name, cfg.unfiltered),
                    "web_dir": web_evidence_dir,
                    "wiki_dir": wiki_evidence_dir,
                },
            )
            for name in [datasets.Split.TRAIN, datasets.Split.VALIDATION, datasets.Split.TEST]
        ]

    def _generate_examples(self, files, web_dir, wiki_dir):
        """This function returns the examples."""

        def parse_example(article):
            """Return a single example from an article JSON record."""

            def _strip(collection):
                return [item.strip() for item in collection]

            if "Answer" in article:
                answer = article["Answer"]
                answer_dict = {
                    "aliases": _strip(answer["Aliases"]),
                    "normalized_aliases": _strip(answer["NormalizedAliases"]),
                    "matched_wiki_entity_name": answer.get("MatchedWikiEntryName", "").strip(),
                    "normalized_matched_wiki_entity_name": answer.get("NormalizedMatchedWikiEntryName", "").strip(),
                    "normalized_value": answer["NormalizedValue"].strip(),
                    "type": answer["Type"].strip(),
                    "value": answer["Value"].strip(),
                }
            else:
                answer_dict = {
                    "aliases": [],
                    "normalized_aliases": [],
                    "matched_wiki_entity_name": "<unk>",
                    "normalized_matched_wiki_entity_name": "<unk>",
                    "normalized_value": "<unk>",
                    "type": "",
                    "value": "<unk>",
                }

            if self.config.exclude_context:
                article["SearchResults"] = []
                article["EntityPages"] = []

            def _add_context(collection, context_field, file_dir):
                """Adds context from file, or skips if file does not exist."""
                new_items = []
                for item in collection:
                    if "Filename" not in item:
                        logger.info("Missing context 'Filename', skipping.")
                        continue

                    new_item = item.copy()
                    fname = item["Filename"]
                    try:
                        with open(os.path.join(file_dir, fname), encoding="utf-8") as f:
                            new_item[context_field] = f.read()
                    except (IOError, FileNotFoundError):
                        logger.info("File does not exist, skipping: %s", fname)
                        continue
                    new_items.append(new_item)
                return new_items

            def _strip_if_str(v):
                return v.strip() if isinstance(v, str) else v

            def _transpose_and_strip_dicts(dicts, field_names):
                return {
                    datasets.naming.camelcase_to_snakecase(k): [_strip_if_str(d[k]) for d in dicts]
                    for k in field_names
                }

            search_results = _transpose_and_strip_dicts(
                _add_context(article.get("SearchResults", []), "SearchContext", web_dir),
                ["Description", "Filename", "Rank", "Title", "Url", "SearchContext"],
            )

            entity_pages = _transpose_and_strip_dicts(
                _add_context(article.get("EntityPages", []), "WikiContext", wiki_dir),
                ["DocSource", "Filename", "Title", "WikiContext"],
            )

            question = article["Question"].strip()
            question_id = article["QuestionId"]
            question_source = article["QuestionSource"].strip()

            return {
                "entity_pages": entity_pages,
                "search_results": search_results,
                "question": question,
                "question_id": question_id,
                "question_source": question_source,
                "answer": answer_dict,
            }

        for filepath in files:
            logger.info("generating examples from = %s", filepath)
            fname = os.path.basename(filepath)

            with open(filepath, encoding="utf-8") as f:
                current_record = ""
                for line in f:
                    if line == "        {\n":
                        current_record = line
                    elif line.startswith("        }"):  # Handles final record as well.
                        article = json.loads(current_record + "}")
                        current_record = ""
                        example = parse_example(article)
                        yield "%s_%s" % (fname, example["question_id"]), example
                    else:
                        current_record += line



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""CVIT PIB Multilingual Corpus"""

import datasets


_CITATION = """\
@inproceedings{siripragada-etal-2020-multilingual,
    title = "A Multilingual Parallel Corpora Collection Effort for {I}ndian Languages",
    author = "Siripragada, Shashank  and
      Philip, Jerin  and
      Namboodiri, Vinay P.  and
      Jawahar, C V",
    booktitle = "Proceedings of the 12th Language Resources and Evaluation Conference",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.lrec-1.462",
    pages = "3743--3751",
    language = "English",
    ISBN = "979-10-95546-34-4",
}
@article{2020,
   title={Revisiting Low Resource Status of Indian Languages in Machine Translation},
   url={http://dx.doi.org/10.1145/3430984.3431026},
   DOI={10.1145/3430984.3431026},
   journal={8th ACM IKDD CODS and 26th COMAD},
   publisher={ACM},
   author={Philip, Jerin and Siripragada, Shashank and Namboodiri, Vinay P. and Jawahar, C. V.},
   year={2020},
   month={Dec}
}
"""

_DESCRIPTION = """\
Sentence aligned parallel corpus between 11 Indian Languages, crawled and extracted from the press information bureau
website.
"""

_HOMEPAGE = "http://preon.iiit.ac.in/~jerin/bhasha/"

_LICENSE = "Creative Commons Attribution-ShareAlike 4.0 International"

_URL = {
    "0.0.0": "http://preon.iiit.ac.in/~jerin/resources/datasets/pib-v0.tar",
    "1.3.0": "http://preon.iiit.ac.in/~jerin/resources/datasets/pib_v1.3.tar.gz",
}
_ROOT_DIR = {
    "0.0.0": "pib",
    "1.3.0": "pib-v1.3",
}

_LanguagePairs = [
    "or-ur",
    "ml-or",
    "bn-ta",
    "gu-mr",
    "hi-or",
    "en-or",
    "mr-ur",
    "en-ta",
    "hi-ta",
    "bn-en",
    "bn-or",
    "ml-ta",
    "gu-ur",
    "bn-ml",
    "ml-pa",
    "en-pa",
    "bn-hi",
    "hi-pa",
    "gu-te",
    "pa-ta",
    "hi-ml",
    "or-te",
    "en-ml",
    "en-hi",
    "bn-pa",
    "mr-te",
    "mr-pa",
    "bn-te",
    "gu-hi",
    "ta-ur",
    "te-ur",
    "or-pa",
    "gu-ml",
    "gu-pa",
    "hi-te",
    "en-te",
    "ml-te",
    "pa-ur",
    "hi-ur",
    "mr-or",
    "en-ur",
    "ml-ur",
    "bn-mr",
    "gu-ta",
    "pa-te",
    "bn-gu",
    "bn-ur",
    "ml-mr",
    "or-ta",
    "ta-te",
    "gu-or",
    "en-gu",
    "hi-mr",
    "mr-ta",
    "en-mr",
]


class PibConfig(datasets.BuilderConfig):
    """BuilderConfig for PIB"""

    def __init__(self, language_pair, version=datasets.Version("1.3.0"), **kwargs):
        super().__init__(version=version, **kwargs)
        """

        Args:
            language_pair: language pair, you want to load
            **kwargs: keyword arguments forwarded to super.
        """
        self.src, self.tgt = language_pair.split("-")


class Pib(datasets.GeneratorBasedBuilder):
    """This new dataset is the large scale sentence aligned corpus in 11 Indian languages, viz.
    CVIT-PIB corpus that is the largest multilingual corpus available for Indian languages.
    """

    BUILDER_CONFIG_CLASS = PibConfig
    BUILDER_CONFIGS = [PibConfig(name=pair, description=_DESCRIPTION, language_pair=pair) for pair in _LanguagePairs]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=[self.config.src, self.config.tgt])}
            ),
            supervised_keys=(self.config.src, self.config.tgt),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        archive = dl_manager.download(_URL[str(self.config.version)])
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "archive": dl_manager.iter_archive(archive),
                },
            ),
        ]

    def _generate_examples(self, archive):
        root_dir = _ROOT_DIR[str(self.config.version)]
        data_dir = f"{root_dir}/{self.config.src}-{self.config.tgt}"
        src = tgt = None
        for path, file in archive:
            if data_dir in path:
                if f"{data_dir}/train.{self.config.src}" in path:
                    src = file.read().decode("utf-8").split("\n")[:-1]
                if f"{data_dir}/train.{self.config.tgt}" in path:
                    tgt = file.read().decode("utf-8").split("\n")[:-1]
            if src and tgt:
                break
        for idx, (s, t) in enumerate(zip(src, tgt)):
            yield idx, {"translation": {self.config.src: s, self.config.tgt: t}}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MENYO-20k: A Multi-domain English - Yorùbá Corpus for Machine Translations"""


import csv

import datasets


# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@dataset{david_ifeoluwa_adelani_2020_4297448,
  author       = {David Ifeoluwa Adelani and
                  Jesujoba O. Alabi and
                  Damilola Adebonojo and
                  Adesina Ayeni and
                  Mofe Adeyemi and
                  Ayodele Awokoya},
  title        = {MENYO-20k: A Multi-domain English - Yorùbá Corpus
                  for Machine Translation},
  month        = nov,
  year         = 2020,
  publisher    = {Zenodo},
  version      = {1.0},
  doi          = {10.5281/zenodo.4297448},
  url          = {https://doi.org/10.5281/zenodo.4297448}
}
"""


# You can copy an official description
_DESCRIPTION = """\
MENYO-20k is a multi-domain parallel dataset with texts obtained from news articles, ted talks, movie transcripts, radio transcripts, science and technology texts, and other short articles curated from the web and professional translators. The dataset has 20,100 parallel sentences split into 10,070 training sentences, 3,397 development sentences, and 6,633 test sentences (3,419 multi-domain, 1,714 news domain, and 1,500 ted talks speech transcript domain). The development and test sets are available upon request.
"""


_HOMEPAGE = "https://zenodo.org/record/4297448#.X81G7s0zZPY"


_LICENSE = "For non-commercial use because some of the data sources like Ted talks and JW news requires permission for commercial use."


# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "https://raw.githubusercontent.com/uds-lsv/menyo-20k_MT/master/data/train.tsv"


class Menyo20kMt(datasets.GeneratorBasedBuilder):
    """MENYO-20k: A Multi-domain English - Yorùbá Corpus for Machine Translations"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="menyo20k_mt",
            version=VERSION,
            description="MENYO-20k: A Multi-domain English - Yorùbá Corpus for Machine Translations",
        )
    ]

    def _info(self):

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=datasets.Features({"translation": datasets.features.Translation(languages=("en", "yo"))}),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        train_path = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            for idx, row in enumerate(reader):
                result = {"translation": {"en": row["English"], "yo": row["Yoruba"]}}
                yield idx, result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cyberbullying Classification Dataset in Polish"""


import os

import datasets


_DESCRIPTION = """\
    In Task 6-1, the participants are to distinguish between normal/non-harmful tweets (class: 0) and tweets
    that contain any kind of harmful information (class: 1). This includes cyberbullying, hate speech and
    related phenomena.

    In Task 6-2, the participants shall distinguish between three classes of tweets: 0 (non-harmful),
    1 (cyberbullying), 2 (hate-speech). There are various definitions of both cyberbullying and hate-speech,
    some of them even putting those two phenomena in the same group. The specific conditions on which we based
    our annotations for both cyberbullying and hate-speech, which have been worked out during ten years of research
    will be summarized in an introductory paper for the task, however, the main and definitive condition to 1
    distinguish the two is whether the harmful action is addressed towards a private person(s) (cyberbullying),
    or a public person/entity/large group (hate-speech).
"""

_HOMEPAGE = "http://2019.poleval.pl/index.php/tasks/task6"

_URL_TRAIN_TASK1 = "http://2019.poleval.pl/task6/task_6-1.zip"
_URL_TRAIN_TASK2 = "http://2019.poleval.pl/task6/task_6-2.zip"
_URL_TEST = "http://2019.poleval.pl/task6/task6_test.zip"

_CITATION = """\
@proceedings{ogr:kob:19:poleval,
  editor    = {Maciej Ogrodniczuk and Łukasz Kobyliński},
  title     = {{Proceedings of the PolEval 2019 Workshop}},
  year      = {2019},
  address   = {Warsaw, Poland},
  publisher = {Institute of Computer Science, Polish Academy of Sciences},
  url       = {http://2019.poleval.pl/files/poleval2019.pdf},
  isbn      = "978-83-63159-28-3"}
}
"""


class Poleval2019CyberBullyingConfig(datasets.BuilderConfig):
    """BuilderConfig for Poleval2019CyberBullying."""

    def __init__(
        self,
        text_features,
        label_classes,
        **kwargs,
    ):
        super(Poleval2019CyberBullyingConfig, self).__init__(version=datasets.Version("1.0.0"), **kwargs)
        self.text_features = text_features
        self.label_classes = label_classes


class Poleval2019CyberBullying(datasets.GeneratorBasedBuilder):
    """Cyberbullying Classification Dataset in Polish"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        Poleval2019CyberBullyingConfig(
            name="task01",
            text_features=["text"],
            label_classes=["0", "1"],
        ),
        Poleval2019CyberBullyingConfig(
            name="task02",
            text_features=["text"],
            label_classes=["0", "1", "2"],
        ),
    ]

    def _info(self):

        features = {text_feature: datasets.Value("string") for text_feature in self.config.text_features}
        features["label"] = datasets.features.ClassLabel(names=self.config.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=("text", "label"),
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        if self.config.name == "task01":
            train_path = dl_manager.download_and_extract(_URL_TRAIN_TASK1)

        if self.config.name == "task02":
            train_path = dl_manager.download_and_extract(_URL_TRAIN_TASK2)

        data_dir_test = dl_manager.download_and_extract(_URL_TEST)

        if self.config.name == "task01":
            test_path = os.path.join(data_dir_test, "Task6", "task 01")

        if self.config.name == "task02":
            test_path = os.path.join(data_dir_test, "Task6", "task 02")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        if split == "train":
            text_path = os.path.join(filepath, "training_set_clean_only_text.txt")
            label_path = os.path.join(filepath, "training_set_clean_only_tags.txt")

        if split == "test":
            if self.config.name == "task01":
                text_path = os.path.join(filepath, "test_set_clean_only_text.txt")
                label_path = os.path.join(filepath, "test_set_clean_only_tags.txt")
            if self.config.name == "task02":
                text_path = os.path.join(filepath, "test_set_only_text.txt")
                label_path = os.path.join(filepath, "test_set_only_tags.txt")

        with open(text_path, encoding="utf-8") as text_file:
            with open(label_path, encoding="utf-8") as label_file:
                for id_, (text, label) in enumerate(zip(text_file, label_file)):
                    yield id_, {"text": text.strip(), "label": int(label.strip())}



"""TODO(cornell_movie_dialog): Add a description here."""


import ast
import os

import datasets


# TODO(cornell_movie_dialog): BibTeX citation
_CITATION = """\
  @InProceedings{Danescu-Niculescu-Mizil+Lee:11a,

  author={Cristian Danescu-Niculescu-Mizil and Lillian Lee},

  title={Chameleons in imagined conversations:
  A new approach to understanding coordination of linguistic style in dialogs.},

  booktitle={Proceedings of the

        Workshop on Cognitive Modeling and Computational Linguistics, ACL 2011},

  year={2011}

}
"""

# TODO(cornell_movie_dialog):
_DESCRIPTION = """\
This corpus contains a large metadata-rich collection of fictional conversations extracted from raw movie scripts:
- 220,579 conversational exchanges between 10,292 pairs of movie characters
- involves 9,035 characters from 617 movies
- in total 304,713 utterances
- movie metadata included:
    - genres
    - release year
    - IMDB rating
    - number of IMDB votes
    - IMDB rating
- character metadata included:
    - gender (for 3,774 characters)
    - position on movie credits (3,321 characters)
"""

_URL = "https://www.cs.cornell.edu/~cristian/data/cornell_movie_dialogs_corpus.zip"


class CornellMovieDialog(datasets.GeneratorBasedBuilder):
    """TODO(cornell_movie_dialog): Short description of my dataset."""

    # TODO(cornell_movie_dialog): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(cornell_movie_dialog): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "movieID": datasets.Value("string"),
                    "movieTitle": datasets.Value("string"),
                    "movieYear": datasets.Value("string"),
                    "movieIMDBRating": datasets.Value("string"),
                    "movieNoIMDBVotes": datasets.Value("string"),
                    "movieGenres": datasets.features.Sequence(datasets.Value("string")),
                    "characterID1": datasets.Value("string"),
                    "characterID2": datasets.Value("string"),
                    "characterName1": datasets.Value("string"),
                    "characterName2": datasets.Value("string"),
                    "utterance": datasets.features.Sequence(
                        {"text": datasets.Value("string"), "LineID": datasets.Value("string")}
                    )
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="http://www.cs.cornell.edu/~cristian/Cornell_Movie-Dialogs_Corpus.html",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(cornell_movie_dialog): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepaths": os.path.join(dl_dir, "cornell movie-dialogs corpus")},
            ),
        ]

    def _generate_examples(self, filepaths):
        """Yields examples."""
        # TODO(cornell_movie_dialog): Yields (key, example) tuples from the dataset
        movie_char_file = os.path.join(filepaths, "movie_characters_metadata.txt")
        movie_conv_file = os.path.join(filepaths, "movie_conversations.txt")
        movie_lines_file = os.path.join(filepaths, "movie_lines.txt")
        movie_titles_file = os.path.join(filepaths, "movie_titles_metadata.txt")

        with open(movie_char_file, "rb") as f:
            movie_char_data = [x.decode("latin").split("+++$+++") for x in f.readlines()]

        with open(movie_conv_file, "rb") as f:
            movie_conv_data = [x.decode("latin").split("+++$+++") for x in f.readlines()]

        with open(movie_lines_file, "rb") as f:
            movie_lines_data = [x.decode("latin").split("+++$+++") for x in f.readlines()]

        with open(movie_titles_file, "rb") as f:
            movie_titles_data = [x.decode("latin").split("+++$+++") for x in f.readlines()]
        # looping over movie conversation file
        for id_, conv in enumerate(movie_conv_data):
            char_id_1 = conv[0]
            char_id_2 = conv[1]
            movie_id = conv[2]
            line_ids = conv[-1].replace("\n", "")
            line_ids = ast.literal_eval(line_ids.strip())
            lines_texts = []
            # searching text corresponding to each lineID in line_ids in movie lines file
            for line_id in line_ids:
                i = 0
                while i < len(movie_lines_data) and movie_lines_data[i][0].strip() != line_id:
                    i += 1
                lines_texts.append(movie_lines_data[i][0])  # if i < len(movie_lines_data) else '')
            # look for char names in movie character file
            j = 0
            while j < len(movie_char_data) and movie_char_data[j][0].strip() != char_id_1.strip():
                j += 1
            char_name_1 = movie_char_data[j][1]  # if j < len(movie_char_data) else ''
            movie_title = movie_char_data[j][3]  # if j < len(movie_char_data) else ''

            k = 0
            while k < len(movie_char_data) and movie_char_data[k][0].strip() != char_id_2.strip():
                k += 1
            char_name_2 = movie_char_data[k][1]

            # look for movie year, IMDBRating, genre, no_imdb_voting in movie tiles file
            li = 0
            while li < len(movie_titles_data) and movie_titles_data[li][0].strip() != movie_id.strip():
                li += 1
            movie_year = movie_titles_data[li][2]
            imdb_rating = movie_titles_data[li][3]
            no_imdb_vote = movie_titles_data[li][4]
            genre = movie_titles_data[li][5].replace("\n", "").strip()
            movie_genres = ast.literal_eval(genre)

            yield id_, {
                "movieID": movie_id,
                "movieTitle": movie_title,
                "movieYear": movie_year,
                "movieIMDBRating": imdb_rating,
                "movieNoIMDBVotes": no_imdb_vote,
                "movieGenres": movie_genres,
                "characterID1": char_id_1,
                "characterID2": char_id_2,
                "characterName1": char_name_1,
                "characterName2": char_name_2,
                "utterance": {"text": lines_texts, "LineID": line_ids},
            }



import os
from functools import reduce
from pathlib import Path

import datasets


_CITATION = """\
@inproceedings{kosawat2009best,
  title={BEST 2009: Thai word segmentation software contest},
  author={Kosawat, Krit and Boriboon, Monthika and Chootrakool, Patcharika and Chotimongkol, Ananlada and Klaithin, Supon and Kongyoung, Sarawoot and Kriengket, Kanyanut and Phaholphinyo, Sitthaa and Purodakananda, Sumonmas and Thanakulwarapas, Tipraporn and others},
  booktitle={2009 Eighth International Symposium on Natural Language Processing},
  pages={83--88},
  year={2009},
  organization={IEEE}
}
@inproceedings{boriboon2009best,
  title={Best corpus development and analysis},
  author={Boriboon, Monthika and Kriengket, Kanyanut and Chootrakool, Patcharika and Phaholphinyo, Sitthaa and Purodakananda, Sumonmas and Thanakulwarapas, Tipraporn and Kosawat, Krit},
  booktitle={2009 International Conference on Asian Language Processing},
  pages={322--327},
  year={2009},
  organization={IEEE}
}
"""

_LICENSE = "CC-BY-NC-SA 3.0"

_DESCRIPTION = """\
`best2009` is a Thai word-tokenization dataset from encyclopedia, novels, news and articles by
[NECTEC](https://www.nectec.or.th/) (148,995/2,252 lines of train/test). It was created for
[BEST 2010: Word Tokenization Competition](https://thailang.nectec.or.th/archive/indexa290.html?q=node/10).
The test set answers are not provided publicly.
"""


class Best2009Config(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        """BuilderConfig

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(Best2009Config, self).__init__(**kwargs)


class Best2009(datasets.GeneratorBasedBuilder):

    _DOWNLOAD_URL = "https://archive.org/download/best_dataset/data.zip"
    _TRAIN_FOLDER = "train"
    _TEST_FOLDER = "test"

    _USELESS_TAGS = {"<NE>": "", "</NE>": "", "<AB>": "", "</AB>": ""}
    # character type mapping from https://github.com/rkcosmos/deepcut/blob/master/deepcut/utils.py
    _CHAR_TYPES_DICT = {
        "กขฃคฆงจชซญฎฏฐฑฒณดตถทธนบปพฟภมยรลวศษสฬอ": "c",
        "ฅฉผฟฌหฮ": "n",
        "ะาำิีืึุู": "v",  # า ะ ำ ิ ี ึ ื ั ู ุ
        "เแโใไ": "w",
        "่้๊๋": "t",  # วรรณยุกต์ ่ ้ ๊ ๋
        "์ๆฯ.": "s",  # ์  ๆ ฯ .
        "0123456789๑๒๓๔๕๖๗๘๙": "d",
        '"': "q",
        "‘": "q",
        "’": "q",
        "'": "q",
        " ": "p",
        "abcdefghijklmnopqrstuvwxyz": "s_e",
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ": "b_e",
    }
    _CHAR_TYPE_FLATTEN = {}
    for ks, v in _CHAR_TYPES_DICT.items():
        for k in ks:
            _CHAR_TYPE_FLATTEN[k] = v
    _CHAR_TYPES = ["b_e", "c", "d", "n", "o", "p", "q", "s", "s_e", "t", "v", "w"]

    BUILDER_CONFIGS = [
        Best2009Config(
            name="best2009",
            version=datasets.Version("1.0.0"),
            description=_DESCRIPTION,
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "fname": datasets.Value("string"),
                    "char": datasets.Sequence(datasets.Value("string")),
                    "char_type": datasets.Sequence(datasets.features.ClassLabel(names=self._CHAR_TYPES)),
                    "is_beginning": datasets.Sequence(datasets.features.ClassLabel(names=["neg", "pos"])),
                }
            ),
            supervised_keys=None,
            homepage="https://aiforthai.in.th/",
            citation=_CITATION,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FOLDER), "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FOLDER), "split": "train"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        for file_idx, fname in enumerate(sorted(Path(filepath).rglob("*.txt"))):
            with open(fname, encoding="utf-8") as f:
                for line_idx, line in enumerate(f):
                    chars = []
                    char_types = []
                    is_beginnings = []
                    # replace useless tokens
                    line = reduce(lambda a, kv: a.replace(*kv), self._USELESS_TAGS.items(), line)
                    # tokens are pipe separated
                    splits = line.split("|")
                    for token in splits:
                        for i in range(len(token)):
                            chars.append(token[i])
                            char_types.append(self._CHAR_TYPE_FLATTEN.get(token[i], "o"))
                            is_beginning = 1 if i == 0 else 0
                            is_beginnings.append(is_beginning)
                    yield f"{file_idx}_{line_idx}", {
                        "fname": fname.name,
                        "char": chars,
                        "char_type": char_types,
                        "is_beginning": is_beginnings if split == "train" else [0 for i in range(len(chars))],
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Swahili : News Classification Dataset"""

import csv

import datasets


_CITATION = """\
@dataset{davis_david_2020_5514203,
  author       = {Davis David},
  title        = {Swahili : News Classification Dataset},
  month        = dec,
  year         = 2020,
  note         = {{The news version contains both train and test sets.}},
  publisher    = {Zenodo},
  version      = {0.2},
  doi          = {10.5281/zenodo.5514203},
  url          = {https://doi.org/10.5281/zenodo.5514203}
}
"""

_DESCRIPTION = """\
Swahili is spoken by 100-150 million people across East Africa. In Tanzania, it is one of two national languages (the other is English) and it is the official language of instruction in all schools. News in Swahili is an important part of the media sphere in Tanzania.

News contributes to education, technology, and the economic growth of a country, and news in local languages plays an important cultural role in many Africa countries. In the modern age, African languages in news and other spheres are at risk of being lost as English becomes the dominant language in online spaces.

The Swahili news dataset was created to reduce the gap of using the Swahili language to create NLP technologies and help AI practitioners in Tanzania and across Africa continent to practice their NLP skills to solve different problems in organizations or societies related to Swahili language. Swahili News were collected from different websites that provide news in the Swahili language. I was able to find some websites that provide news in Swahili only and others in different languages including Swahili.

The dataset was created for a specific task of text classification, this means each news content can be categorized into six different topics (Local news, International news , Finance news, Health news, Sports news, and Entertainment news). The dataset comes with a specified train/test split. The train set contains 75% of the dataset and test set contains 25% of the dataset.
"""

_HOMEPAGE = "https://zenodo.org/record/4300294#.X84BQdgzZPb"

_LICENSE = "Creative Commons Attribution 4.0 International"

# The HuggingFace Datasets library don't host the datasets but only point to the original files
_URLS = {
    "train": "https://zenodo.org/record/5514203/files/train_v0.2.csv?download=1",
    "test": "https://zenodo.org/record/5514203/files/test_v0.2.csv?download=1",
}


class SwahiliNews(datasets.GeneratorBasedBuilder):
    """Swahili : News Classification Dataset"""

    VERSION = datasets.Version("0.2.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="swahili_news",
            version=VERSION,
            description="Swahili : News Classification Dataset",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(
                        names=["uchumi", "kitaifa", "michezo", "kimataifa", "burudani", "afya"]
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        paths = dl_manager.download_and_extract(_URLS)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": paths["train"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": paths["test"]}),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.DictReader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            for id_, row in enumerate(csv_reader):
                yield id_, row



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""The Stanford Natural Language Inference (SNLI) Corpus."""


import csv
import os

import datasets


_CITATION = """\
@inproceedings{snli:emnlp2015,
    Author = {Bowman, Samuel R. and Angeli, Gabor and Potts, Christopher, and Manning, Christopher D.},
    Booktitle = {Proceedings of the 2015 Conference on Empirical Methods in Natural Language Processing (EMNLP)},
    Publisher = {Association for Computational Linguistics},
    Title = {A large annotated corpus for learning natural language inference},
    Year = {2015}
}
"""

_DESCRIPTION = """\
The SNLI corpus (version 1.0) is a collection of 570k human-written English
sentence pairs manually labeled for balanced classification with the labels
entailment, contradiction, and neutral, supporting the task of natural language
inference (NLI), also known as recognizing textual entailment (RTE).
"""

_DATA_URL = "https://nlp.stanford.edu/projects/snli/snli_1.0.zip"


class Snli(datasets.GeneratorBasedBuilder):
    """The Stanford Natural Language Inference (SNLI) Corpus."""

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            version=datasets.Version("1.0.0", ""),
            description="Plain text import of SNLI",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "hypothesis": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["entailment", "neutral", "contradiction"]),
                }
            ),
            # No default supervised_keys (as we have to pass both premise
            # and hypothesis as input).
            supervised_keys=None,
            homepage="https://nlp.stanford.edu/projects/snli/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DATA_URL)
        data_dir = os.path.join(dl_dir, "snli_1.0")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"filepath": os.path.join(data_dir, "snli_1.0_test.txt")}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepath": os.path.join(data_dir, "snli_1.0_dev.txt")}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(data_dir, "snli_1.0_train.txt")}
            ),
        ]

    def _generate_examples(self, filepath):
        """This function returns the examples in the raw (text) form."""
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            for idx, row in enumerate(reader):
                label = -1 if row["gold_label"] == "-" else row["gold_label"]
                yield idx, {
                    "premise": row["sentence1"],
                    "hypothesis": row["sentence2"],
                    "label": label,
                }



"""TODO(empathetic_dialogues): Add a description here."""


import csv

import datasets


_CITATION = """\
@inproceedings{rashkin2019towards,
  title = {Towards Empathetic Open-domain Conversation Models: a New Benchmark and Dataset},
  author = {Hannah Rashkin and Eric Michael Smith and Margaret Li and Y-Lan Boureau},
  booktitle = {ACL},
  year = {2019},
}
"""

_DESCRIPTION = """\
PyTorch original implementation of Towards Empathetic Open-domain Conversation Models: a New Benchmark and Dataset
"""
_URL = "https://dl.fbaipublicfiles.com/parlai/empatheticdialogues/empatheticdialogues.tar.gz"


class EmpatheticDialogues(datasets.GeneratorBasedBuilder):
    """TODO(empathetic_dialogues): Short description of my dataset."""

    # TODO(empathetic_dialogues): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(empathetic_dialogues): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "conv_id": datasets.Value("string"),
                    "utterance_idx": datasets.Value("int32"),
                    "context": datasets.Value("string"),
                    "prompt": datasets.Value("string"),
                    "speaker_idx": datasets.Value("int32"),
                    "utterance": datasets.Value("string"),
                    "selfeval": datasets.Value("string"),
                    "tags": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/facebookresearch/EmpatheticDialogues",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(empathetic_dialogues): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        archive = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"files": dl_manager.iter_archive(archive), "split_file": "empatheticdialogues/train.csv"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"files": dl_manager.iter_archive(archive), "split_file": "empatheticdialogues/valid.csv"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"files": dl_manager.iter_archive(archive), "split_file": "empatheticdialogues/test.csv"},
            ),
        ]

    def _generate_examples(self, files, split_file):
        """Yields examples."""
        for path, f in files:
            if split_file == path:
                data = csv.DictReader(line.decode("utf-8") for line in f)
                for id_, row in enumerate(data):
                    utterance = row["utterance"]
                    speaker_id = int(row["speaker_idx"])
                    context = row["context"]
                    conv_id = row["conv_id"]
                    tags = row["tags"] if row["tags"] else ""
                    selfeval = row["selfeval"] if row["selfeval"] else ""
                    utterance_id = int(row["utterance_idx"])
                    prompt = row["prompt"]
                    yield id_, {
                        "utterance": utterance,
                        "utterance_idx": utterance_id,
                        "context": context,
                        "speaker_idx": speaker_id,
                        "conv_id": conv_id,
                        "selfeval": selfeval,
                        "prompt": prompt,
                        "tags": tags,
                    }
                break



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Introduction to the Bio-Entity Recognition Task at JNLPBA"""

import glob
import os
import re

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{kim2004introduction,
               title={Introduction to the bio-entity recognition task at JNLPBA},
               author={Kim, Jin-Dong and Ohta, Tomoko and Tsuruoka, Yoshimasa and Tateisi, Yuka and Collier, Nigel},
               booktitle={Proceedings of the international joint workshop on natural language processing in biomedicine and its applications},
               pages={70--75},
               year={2004},
               organization={Citeseer}
}
"""

_DESCRIPTION = """\
The data came from the GENIA version 3.02 corpus (Kim et al., 2003). This was formed from a controlled search
on MEDLINE using the MeSH terms human, blood cells and transcription factors. From this search 2,000 abstracts
were selected and hand annotated according to a small taxonomy of 48 classes based on a chemical classification.
Among the classes, 36 terminal classes were used to annotate the GENIA corpus.
"""

_HOMEPAGE = "http://www.geniaproject.org/shared-tasks/bionlp-jnlpba-shared-task-2004"
TRAIN_URL = "http://www.nactem.ac.uk/GENIA/current/Shared-tasks/JNLPBA/Train/Genia4ERtraining.tar.gz"
VAL_URL = "http://www.nactem.ac.uk/GENIA/current/Shared-tasks/JNLPBA/Evaluation/Genia4ERtest.tar.gz"


class JNLPBAConfig(datasets.BuilderConfig):
    """BuilderConfig for JNLPBA"""

    def __init__(self, **kwargs):
        """BuilderConfig for JNLPBA.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(JNLPBAConfig, self).__init__(**kwargs)


class JNLPBA(datasets.GeneratorBasedBuilder):
    """JNLPBA dataset."""

    BUILDER_CONFIGS = [
        JNLPBAConfig(name="jnlpba", version=datasets.Version("1.0.0"), description="JNLPBA dataset"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-DNA",
                                "I-DNA",
                                "B-RNA",
                                "I-RNA",
                                "B-cell_line",
                                "I-cell_line",
                                "B-cell_type",
                                "I-cell_type",
                                "B-protein",
                                "I-protein",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        train_files = dl_manager.download_and_extract(TRAIN_URL)
        val_files = dl_manager.download_and_extract(VAL_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_files}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": val_files}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        filenames = glob.glob(os.path.join(filepath, "Genia4ER*.iob2"))
        guid = 0
        for filename in filenames:
            with open(filename, encoding="utf-8") as f:
                if guid >= 0:
                    guid += 1  # update guid to avoid DuplicatedKeysError
                tokens = []
                ner_tags = []
                for line in f:
                    if len(re.split(r"###MEDLINE:", line)) == 2:
                        continue

                    elif line == "" or line == "\n":
                        if tokens:
                            # print(guid, line)
                            yield guid, {
                                "id": str(guid),
                                "tokens": tokens,
                                "ner_tags": ner_tags,
                            }
                            guid += 1
                            tokens = []
                            ner_tags = []

                    else:
                        # tokens are tab separated
                        splits = line.split("\t")
                        tokens.append(splits[0])
                        ner_tags.append(splits[1].rstrip())
                # last example
                yield guid, {
                    "id": str(guid),
                    "tokens": tokens,
                    "ner_tags": ner_tags,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PolEmo2.0 IN and OUT"""


import csv
import os

import datasets


_CITATION = """\
@inproceedings{kocon-etal-2019-multi,
title = "Multi-Level Sentiment Analysis of {P}ol{E}mo 2.0: Extended Corpus of Multi-Domain Consumer Reviews",
author = "Koco{\'n}, Jan and
Milkowski, Piotr and
Za{\'s}ko-Zieli{\'n}ska, Monika",
booktitle = "Proceedings of the 23rd Conference on Computational Natural Language Learning (CoNLL)",
month = nov,
year = "2019",
address = "Hong Kong, China",
publisher = "Association for Computational Linguistics",
url = "https://www.aclweb.org/anthology/K19-1092",
doi = "10.18653/v1/K19-1092",
pages = "980--991",
}
"""

_DESCRIPTION = """\
The PolEmo2.0 is a set of online reviews from medicine and hotels domains. The task is to predict the sentiment of a review. There are two separate test sets, to allow for in-domain (medicine and hotels) as well as out-of-domain (products and university) validation.
"""

_HOMEPAGE = "https://clarin-pl.eu/dspace/handle/11321/710"

_LICENSE = "CC BY-NC-SA 4.0"

_URLs = {
    "in": "https://klejbenchmark.com/static/data/klej_polemo2.0-in.zip",
    "out": "https://klejbenchmark.com/static/data/klej_polemo2.0-out.zip",
}


class Polemo2(datasets.GeneratorBasedBuilder):
    """PolEmo2.0"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="in",
            version=VERSION,
            description="The PolEmo2.0 is a set of online reviews from medicine and hotels domains. The task is to predict the sentiment of a review. There are two separate test sets, to allow for in-domain (medicine and hotels) as well as out-of-domain (products and university) validation.",
        ),
        datasets.BuilderConfig(
            name="out",
            version=VERSION,
            description="The PolEmo2.0 is a set of online reviews from medicine and hotels domains. The task is to predict the sentiment of a review. There are two separate test sets, to allow for in-domain (medicine and hotels) as well as out-of-domain (products and university) validation.",
        ),
    ]

    DEFAULT_CONFIG_NAME = "in"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "target": datasets.ClassLabel(
                        names=[
                            "__label__meta_amb",
                            "__label__meta_minus_m",
                            "__label__meta_plus_m",
                            "__label__meta_zero",
                        ]
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.tsv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "test_features.tsv"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "dev.tsv"),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            for id_, row in enumerate(reader):
                yield id_, {
                    "sentence": row["sentence"],
                    "target": -1 if split == "test" else row["target"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import glob
import os

import pandas as pd

import datasets


_CITATION = """\
@misc{friedrich2020sofcexp,
      title={The SOFC-Exp Corpus and Neural Approaches to Information Extraction in the Materials Science Domain},
      author={Annemarie Friedrich and Heike Adel and Federico Tomazic and Johannes Hingerl and Renou Benteau and Anika Maruscyk and Lukas Lange},
      year={2020},
      eprint={2006.03039},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
The SOFC-Exp corpus consists of 45 open-access scholarly articles annotated by domain experts.
A corpus and an inter-annotator agreement study demonstrate the complexity of the suggested
named entity recognition and slot filling tasks as well as high annotation quality is presented
in the accompanying paper.
"""

_HOMEPAGE = "https://arxiv.org/abs/2006.03039"

_LICENSE = ""

_URL = "https://github.com/boschresearch/sofc-exp_textmining_resources/archive/master.zip"


class SOFCMaterialsArticles(datasets.GeneratorBasedBuilder):
    """ """

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "text": datasets.Value("string"),
                "sentence_offsets": datasets.features.Sequence(
                    {"begin_char_offset": datasets.Value("int64"), "end_char_offset": datasets.Value("int64")}
                ),
                "sentences": datasets.features.Sequence(datasets.Value("string")),
                "sentence_labels": datasets.features.Sequence(datasets.Value("int64")),
                "token_offsets": datasets.features.Sequence(
                    {
                        "offsets": datasets.features.Sequence(
                            {"begin_char_offset": datasets.Value("int64"), "end_char_offset": datasets.Value("int64")}
                        )
                    }
                ),
                "tokens": datasets.features.Sequence(datasets.features.Sequence(datasets.Value("string"))),
                "entity_labels": datasets.features.Sequence(
                    datasets.features.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "B-DEVICE",
                                "B-EXPERIMENT",
                                "B-MATERIAL",
                                "B-VALUE",
                                "I-DEVICE",
                                "I-EXPERIMENT",
                                "I-MATERIAL",
                                "I-VALUE",
                                "O",
                            ]
                        )
                    )
                ),
                "slot_labels": datasets.features.Sequence(
                    datasets.features.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "B-anode_material",
                                "B-cathode_material",
                                "B-conductivity",
                                "B-current_density",
                                "B-degradation_rate",
                                "B-device",
                                "B-electrolyte_material",
                                "B-experiment_evoking_word",
                                "B-fuel_used",
                                "B-interlayer_material",
                                "B-interconnect_material",
                                "B-open_circuit_voltage",
                                "B-power_density",
                                "B-resistance",
                                "B-support_material",
                                "B-thickness",
                                "B-time_of_operation",
                                "B-voltage",
                                "B-working_temperature",
                                "I-anode_material",
                                "I-cathode_material",
                                "I-conductivity",
                                "I-current_density",
                                "I-degradation_rate",
                                "I-device",
                                "I-electrolyte_material",
                                "I-experiment_evoking_word",
                                "I-fuel_used",
                                "I-interlayer_material",
                                "I-interconnect_material",
                                "I-open_circuit_voltage",
                                "I-power_density",
                                "I-resistance",
                                "I-support_material",
                                "I-thickness",
                                "I-time_of_operation",
                                "I-voltage",
                                "I-working_temperature",
                                "O",
                            ]
                        )
                    )
                ),
                "links": datasets.Sequence(
                    {
                        "relation_label": datasets.features.ClassLabel(
                            names=["coreference", "experiment_variation", "same_experiment", "thickness"]
                        ),
                        "start_span_id": datasets.Value("int64"),
                        "end_span_id": datasets.Value("int64"),
                    }
                ),
                "slots": datasets.features.Sequence(
                    {
                        "frame_participant_label": datasets.features.ClassLabel(
                            names=[
                                "anode_material",
                                "cathode_material",
                                "current_density",
                                "degradation_rate",
                                "device",
                                "electrolyte_material",
                                "fuel_used",
                                "interlayer_material",
                                "open_circuit_voltage",
                                "power_density",
                                "resistance",
                                "support_material",
                                "time_of_operation",
                                "voltage",
                                "working_temperature",
                            ]
                        ),
                        "slot_id": datasets.Value("int64"),
                    }
                ),
                "spans": datasets.features.Sequence(
                    {
                        "span_id": datasets.Value("int64"),
                        "entity_label": datasets.features.ClassLabel(names=["", "DEVICE", "MATERIAL", "VALUE"]),
                        "sentence_id": datasets.Value("int64"),
                        "experiment_mention_type": datasets.features.ClassLabel(
                            names=["", "current_exp", "future_work", "general_info", "previous_work"]
                        ),
                        "begin_char_offset": datasets.Value("int64"),
                        "end_char_offset": datasets.Value("int64"),
                    }
                ),
                "experiments": datasets.features.Sequence(
                    {
                        "experiment_id": datasets.Value("int64"),
                        "span_id": datasets.Value("int64"),
                        "slots": datasets.features.Sequence(
                            {
                                "frame_participant_label": datasets.features.ClassLabel(
                                    names=[
                                        "anode_material",
                                        "cathode_material",
                                        "current_density",
                                        "degradation_rate",
                                        "conductivity",
                                        "device",
                                        "electrolyte_material",
                                        "fuel_used",
                                        "interlayer_material",
                                        "open_circuit_voltage",
                                        "power_density",
                                        "resistance",
                                        "support_material",
                                        "time_of_operation",
                                        "voltage",
                                        "working_temperature",
                                    ]
                                ),
                                "slot_id": datasets.Value("int64"),
                            }
                        ),
                    }
                ),
            }
        )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URL
        data_dir = dl_manager.download_and_extract(my_urls)

        data_dir = os.path.join(data_dir, "sofc-exp_textmining_resources-master/sofc-exp-corpus")

        metadata = pd.read_csv(os.path.join(data_dir, "SOFC-Exp-Metadata.csv"), sep="\t")

        text_base_path = os.path.join(data_dir, "texts")

        text_files_available = [
            os.path.split(i.rstrip(".txt"))[-1] for i in glob.glob(os.path.join(text_base_path, "*.txt"))
        ]

        metadata = metadata[metadata["name"].map(lambda x: x in text_files_available)]

        names = {}
        splits = ["train", "test", "dev"]
        for split in splits:
            names[split] = metadata[metadata["set"] == split]["name"].tolist()

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "names": names["train"],
                    "data_dir": data_dir,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"names": names["test"], "data_dir": data_dir, "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "names": names["dev"],
                    "data_dir": data_dir,
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, names, data_dir, split):
        """Yields examples."""
        # The dataset consists of the original article text as well as annotations
        textfile_base_path = os.path.join(data_dir, "texts")
        annotations_base_path = os.path.join(data_dir, "annotations")

        # The annotations are mostly references to offsets in the source text
        # with corresponding labels, so we'll refer to them as `meta`
        sentence_meta_base_path = os.path.join(annotations_base_path, "sentences")
        tokens_meta_base_path = os.path.join(annotations_base_path, "tokens")
        ets_meta_base_path = os.path.join(annotations_base_path, "entity_types_and_slots")
        frame_meta_base_path = os.path.join(annotations_base_path, "frames")

        # Define the headers for the sentence and token and entity metadata
        sentence_meta_header = ["sentence_id", "label", "begin_char_offset", "end_char_offset"]
        tokens_meta_header = ["sentence_id", "token_id", "begin_char_offset", "end_char_offset"]
        ets_meta_header = [
            "sentence_id",
            "token_id",
            "begin_char_offset",
            "end_char_offset",
            "entity_label",
            "slot_label",
        ]

        # Start the processing loop
        # For each text file, we'll load all of the
        # associated annotation files
        for id_, name in enumerate(sorted(names)):

            # Load the main source text
            textfile_path = os.path.join(textfile_base_path, name + ".txt")
            text = open(textfile_path, encoding="utf-8").read()

            # Load the sentence offsets file
            sentence_meta_path = os.path.join(sentence_meta_base_path, name + ".csv")
            sentence_meta = pd.read_csv(sentence_meta_path, sep="\t", names=sentence_meta_header)

            # Load the tokens offsets file
            tokens_meta_path = os.path.join(tokens_meta_base_path, name + ".csv")
            tokens_meta = pd.read_csv(tokens_meta_path, sep="\t", names=tokens_meta_header)

            # Load the entity offsets file
            ets_meta_path = os.path.join(ets_meta_base_path, name + ".csv")
            ets_meta = pd.read_csv(ets_meta_path, sep="\t", names=ets_meta_header)

            # Create a list of lists indexed as [sentence][token] for the entity and slot labels
            entity_labels = ets_meta.groupby("sentence_id").apply(lambda x: x["entity_label"].tolist()).to_list()
            slot_labels = ets_meta.groupby("sentence_id").apply(lambda x: x["slot_label"].tolist()).to_list()

            # Create a list of lists for the token offsets indexed as [sentence][token]
            # Each element will contain a dict with beginning and ending character offsets
            token_offsets = (
                tokens_meta.groupby("sentence_id")[["begin_char_offset", "end_char_offset"]]
                .apply(lambda x: x.to_dict(orient="records"))
                .tolist()
            )

            # Load the frames metadata. The frames file contains the data for all of the annotations
            # in a condensed format that varies throughout the file. More information on this format
            # can be found: https://framenet.icsi.berkeley.edu/fndrupal/
            frames_meta_path = os.path.join(frame_meta_base_path, name + ".csv")
            frames_meta = open(frames_meta_path, encoding="utf-8").readlines()

            # Parse the sentence offsets, producing a list of dicts with the
            # starting and ending position of each sentence in the original text
            sentence_offsets = (
                sentence_meta[["begin_char_offset", "end_char_offset"]].apply(lambda x: x.to_dict(), axis=1).tolist()
            )

            # The sentence labels are a binary label that describes whether the sentence contains
            # any annotations
            sentence_labels = sentence_meta["label"].tolist()

            # Materialiaze a list of strings of the actual sentences
            sentences = [text[ost["begin_char_offset"] : ost["end_char_offset"]] for ost in sentence_offsets]

            # Materialize a list of lists of the tokens in each sentence.
            # Annotation labels are aligned with these tokens, so be careful with
            # alignment if using your own tokenization scheme with the sentences above
            tokens = [
                [s[tto["begin_char_offset"] : tto["end_char_offset"]] for tto in to]
                for s, to in zip(sentences, token_offsets)
            ]

            # The frames file first contains spans annotations (in one format),
            # then contains experiments annotations (in another format),
            # then links annotations (in yet another format).
            # Here we find the beginning of the experiments and links sections of the file
            # Additionally, each experiment annotation in the experiment annotations begins with a
            # line starting with the word EXPERIMENT (in one format)
            # followed by the annotations for that experiment (in yet _another_ format)
            # Here we get the start positions for each experiment _within_ the experiments
            # section of the frames data
            experiment_starts = [i for i, line in enumerate(frames_meta) if line.startswith("EXPERIMENT")]
            experiment_start = min(experiment_starts)
            link_start = min([i for i, line in enumerate(frames_meta) if line.startswith("LINK")])

            # Pick out the spans section of the data for parsing
            spans_raw = frames_meta[:experiment_start]

            # Iterate through the spans data
            spans = []
            for span in spans_raw:

                # Split out the elements in each tab-delimited line
                _, span_id, entity_label_or_exp, sentence_id, begin_char_offset, end_char_offset = span.split("\t")

                # The entity label for experiment spans have a sub-label,
                # called the experiment mention type,
                # which is sub-delimited by a ':'
                # The code below standardizes the fields produced by
                # each line to a common schema, some fields of which may
                # be empty depending on the data available in the line
                if entity_label_or_exp.startswith("EXPERIMENT"):
                    exp, experiment_mention_type = entity_label_or_exp.split(":")
                    entity_label = ""
                else:
                    entity_label = entity_label_or_exp
                    exp = ""
                    experiment_mention_type = ""

                s = {
                    "span_id": span_id,
                    "entity_label": entity_label,
                    "sentence_id": sentence_id,
                    "experiment_mention_type": experiment_mention_type,
                    "begin_char_offset": int(begin_char_offset),
                    "end_char_offset": int(end_char_offset),
                }
                spans.append(s)

            # Pull out the links annotations for from the frames data
            links_raw = [f.rstrip("\n") for f in frames_meta[link_start:]]

            # Iterate through the links data, which is in a simple tab-delimited format
            links = []
            for link in links_raw:
                _, relation_label, start_span_id, end_span_id = link.split("\t")

                link_out = {
                    "relation_label": relation_label,
                    "start_span_id": int(start_span_id),
                    "end_span_id": int(end_span_id),
                }
                links.append(link_out)

            # Iterate through the experiments data and parse each experiment
            experiments = []
            # Zip the experiment start offsets to get start/end position tuples
            # for each experiment in the experiments data
            for start, end in zip(experiment_starts[:-1], experiment_starts[1:]):
                current_experiment = frames_meta[start:end]
                # The first line of each experiment annotation contains the
                # experiment id and the span id
                _, experiment_id, span_id = current_experiment[0].rstrip("\n").split("\t")
                exp = {"experiment_id": int(experiment_id), "span_id": int(span_id)}

                # The remaining lines in the experiment annotations contain
                # slot level information for each experiment.
                slots = []
                for e in current_experiment[1:]:
                    e = e.rstrip("\n")
                    _, frame_participant_label, slot_id = e.split("\t")
                    to_add = {"frame_participant_label": frame_participant_label, "slot_id": int(slot_id)}
                    slots.append(to_add)
                exp["slots"] = slots

                experiments.append(exp)

            # Yield the final parsed example output
            # NOTE: the `token_offsets` is converted to a list of
            # dicts to accommodate processing to the arrow files
            # in the `features` schema defined above
            yield id_, {
                "text": text,
                "sentence_offsets": sentence_offsets,
                "sentences": sentences,
                "sentence_labels": sentence_labels,
                "token_offsets": [{"offsets": to} for to in token_offsets],
                "tokens": tokens,
                "entity_labels": entity_labels,
                "slot_labels": slot_labels,
                "links": links,
                "slots": slots,
                "spans": spans,
                "experiments": experiments,
            }
